Package: Statial
Type: Package
Title: A package to identify changes in cell state relative to spatial
        associations
Version: 1.15.0
Date: 2025-10-14
Authors@R: c(person("Farhan", "Ameen", email = "fame2827@uni.sydney.edu.au",
    role = c("aut", "cre")),
    person("Sourish", "Iyengar", email = "sourish.iyengar@sydney.edu.au",
    role = c("aut")),
    person("Alex", "Qin", email = "alex.qin@sydney.edu.au",
    role = c("aut")),
    person("Shila", "Ghazanfar", email = "shila.ghazanfar@sydney.edu.au",
    role = c("aut")),
    person("Ellis", "Patrick", email = "ellis.patrick@sydney.edu.au",
    role = c("aut")))
VignetteBuilder: knitr
Encoding: UTF-8
biocViews: SingleCell, Spatial, Classification
Depends: R (>= 4.1.0)
Imports: BiocParallel, spatstat.geom, concaveman, data.table,
        spatstat.explore, dplyr, tidyr, SingleCellExperiment, tibble,
        stringr, tidyselect, ggplot2, methods, stats,
        SummarizedExperiment, S4Vectors, plotly, purrr, ranger,
        magrittr, limma, SpatialExperiment, cluster, treekoR, edgeR
Suggests: BiocStyle, knitr, testthat (>= 3.0.0), ClassifyR, spicyR,
        ggsurvfit, lisaClust, survival
Description: Statial is a suite of functions for identifying changes in cell 
    state. The functionality provided by Statial provides robust quantification 
    of cell type localisation which are invariant to changes in tissue 
    structure. In addition to this Statial uncovers changes in marker expression
    associated with varying levels of localisation. These features can be used 
    to explore how the structure and function of different cell types may be 
    altered by the agents they are surrounded with.
License: GPL-3
RoxygenNote: 7.3.3
Config/testthat/edition: 3
URL: https://sydneybiox.github.io/Statial
        https://github.com/SydneyBioX/Statial/issues
BugReports: https://github.com/SydneyBioX/Statial/issues
git_url: https://git.bioconductor.org/packages/Statial
git_branch: devel
git_last_commit: 97ee04a
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:34:29 UTC; biocbuild
Author: Farhan Ameen [aut, cre],
  Sourish Iyengar [aut],
  Alex Qin [aut],
  Shila Ghazanfar [aut],
  Ellis Patrick [aut]
Maintainer: Farhan Ameen <fame2827@uni.sydney.edu.au>

# Generated by roxygen2: do not edit by hand

export(Kontextual)
export(calcContamination)
export(calcStateChanges)
export(getAbundances)
export(getDistances)
export(getMarkerMeans)
export(getParentPhylo)
export(isKontextual)
export(kontextCurve)
export(kontextPlot)
export(makeWindow)
export(parentCombinations)
export(plotStateChanges)
export(prepMatrix)
export(relabel)
export(relabelKontextual)
import(concaveman)
import(dplyr)
import(spatstat.geom)
import(tidyr)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,SnowParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpmapply)
importFrom(BiocParallel,bpparam)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,metadata)
importFrom(SingleCellExperiment,"reducedDim<-")
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,colData)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(data.table,dcast)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,contains)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,starts_with)
importFrom(dplyr,summarise)
importFrom(dplyr,summarise_at)
importFrom(dplyr,vars)
importFrom(edgeR,DGEList)
importFrom(edgeR,glmQLFTest)
importFrom(edgeR,glmQLFit)
importFrom(edgeR,topTags)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_colour_gradientn)
importFrom(ggplot2,scale_fill_distiller)
importFrom(ggplot2,stat_density_2d)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(limma,lmFit)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(plotly,ggplotly)
importFrom(purrr,reduce)
importFrom(ranger,ranger)
importFrom(spatstat.geom,closepairs)
importFrom(spatstat.geom,owin)
importFrom(spatstat.geom,ppp)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stringr,str_detect)
importFrom(stringr,str_replace)
importFrom(tibble,column_to_rownames)
importFrom(tibble,remove_rownames)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,gather)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyselect,starts_with)

Statial
======================================================

<img src="https://raw.githubusercontent.com/SydneyBioX/SydneyBioXStickers/main/Statial/statial.png" align="right" width=250 style="margin-left: 10px;">

Overview
--------

**Statial** is a suite of functions for identifying changes in cell state. 
The functionality provided by Statial provides robust quantification of cell 
type localisation which are invariant to changes in tissue structure. In 
addition to this Statial uncovers changes in marker expression associated with
varying levels of localisation. These features can be used to explore how the
structure and function of different cell types may be altered by the agents 
they are surrounded with.


Installation
--------
Installing the package from Bioconductor.
```r
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("Statial")
```

Otherwise, install the development version from GitHub.

```r
# Install the development version from GitHub:
# install.packages("devtools")
devtools::install_github("SydneyBioX/Statial")
library(Statial)
```

### Submitting an issue or feature request

`Statial` is still under active development. We would greatly appreciate any and 
all feedback related to the package.

* R package related issues should be raised [here](https://github.com/SydneyBioX/Statial/issues).
* For general questions and feedback, please contact us directly via [ellis.patrick@sydney.edu.au](mailto:ellis.patrick@sydney.edu.au).


## Authors

* **Farhan Ameen** - [Linkedin](https://www.linkedin.com/in/farhan-ameen-5913841b0/)
* **Sourish Iyengar**
* **Alex Qin** - [Linkedin](https://www.linkedin.com/in/alex-qin-5144a7155/)
* **Shila Ghazanfar** - [@shazanfar](https://twitter.com/shazanfar)
* **Ellis Patrick**  - [@TheEllisPatrick](https://twitter.com/TheEllisPatrick)


## Kontextual pre-print

<div class="oxford-citation-text">
Farhan Ameen, Nicholas Robertson, David M. Lin, Shila Ghazanfar, Ellis Patrick
bioRxiv 2024.09.03.611109; doi: https://doi.org/10.1101/2024.09.03.611109
</div>

params <-
list(test = FALSE)

## ----chunk setup, include = FALSE---------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  cache = TRUE,
  message = FALSE,
  warning = FALSE
)
library(BiocStyle)

## ----installation, eval = FALSE-----------------------------------------------
# # Install the package from Bioconductor
# if (!requireNamespace("BiocManager", quietly = TRUE)) {
#   install.packages("BiocManager")
# }
# BiocManager::install("Statial")

## ----libraries and setup, warning = FALSE, message = FALSE, eval = TRUE-------
# Loading required packages
library(Statial)
library(spicyR)
library(ClassifyR)
library(lisaClust)
library(dplyr)
library(SingleCellExperiment)
library(ggplot2)
library(ggsurvfit)
library(survival)
library(tibble)
library(treekoR)

theme_set(theme_classic())
nCores <- 1

## ----load dataset-------------------------------------------------------------
# Load breast cancer
data("kerenSCE")

kerenSCE

## ----biological hierarchy-----------------------------------------------------
# Examine all cell types in image
unique(kerenSCE$cellType)

# Named list of parents and their child cell types
biologicalHierarchy = list(
  "tumour" = c("Keratin_Tumour", "Tumour"),
  "tcells" = c("CD3_Cell", "CD4_Cell", "CD8_Cell", "Tregs"),
  "myeloid" = c("Dc/Mono", "DC", "Mono/Neu", "Macrophages", "Neutrophils"),
  "tissue" = c("Endothelial", "Mesenchymal")
)

# Adding more broader immune parent populationse
biologicalHierarchy$immune = c(biologicalHierarchy$bcells,
                               biologicalHierarchy$tcells,
                               biologicalHierarchy$myeloid,
                               "NK", "other immune", "B")


# Creating a vector for all cellTypes
all <- unique(kerenSCE$cellType)

## ----clustering hierarchy, warning = FALSE------------------------------------
# Calculate hierarchy using treekoR
kerenTree <- treekoR::getClusterTree(t(assay(kerenSCE, "intensities")),
                            kerenSCE$cellType,
                            hierarchy_method="hopach",
                            hopach_K = 1)

# Convert treekoR result to a name list of parents and children.
treekorParents = getParentPhylo(kerenTree)

treekorParents

## ----image6-------------------------------------------------------------------
# Lets define a new cell type vector
kerenSCE$cellTypeNew <- kerenSCE$cellType

# Select for all cells that express higher than baseline level of p53
p53Pos <- assay(kerenSCE)["p53", ] > -0.300460

# Find p53+ tumour cells
kerenSCE$cellTypeNew[kerenSCE$cellType %in% biologicalHierarchy$tumour] <- "Tumour"
kerenSCE$cellTypeNew[p53Pos & kerenSCE$cellType %in% biologicalHierarchy$tumour] <- "p53_Tumour"

# Group all immune cells under the name "Immune"
kerenSCE$cellTypeNew[kerenSCE$cellType %in% biologicalHierarchy$immune] <- "Immune"

# Plot image 6
kerenSCE |>
  colData() |>
  as.data.frame() |>
  filter(imageID == "6") |>
  filter(cellTypeNew %in% c("Immune", "Tumour", "p53_Tumour")) |>
  arrange(cellTypeNew) |>
  ggplot(aes(x = x, y = y, color = cellTypeNew)) +
  geom_point(size = 1) +
  scale_colour_manual(values = c("Immune" = "#505050", "p53_Tumour" = "#64BC46", "Tumour" = "#D6D6D6")) +
  guides(colour = guide_legend(title = "Cell types", override.aes = list(size = 3)))

## ----p53 relationship---------------------------------------------------------
p53_Kontextual <- Kontextual(
  cells = kerenSCE,
  r = 100,
  from = "Immune",
  to = "p53_Tumour",
  parent = c("p53_Tumour", "Tumour"),
  cellType = "cellTypeNew"
)

p53_Kontextual

## ----kontextCurve-------------------------------------------------------------
curves <- kontextCurve(
  cells = kerenSCE,
  from = "Immune",
  to = "p53_Tumour",
  parent = c("p53_Tumour", "Tumour"),
  rs = seq(50, 510, 50),
  image = "6",
  cellType = "cellTypeNew",
  cores = nCores
)

kontextPlot(curves)

## ----parentDf-----------------------------------------------------------------
# Get all relationships between cell types and their parents
parentDf <- parentCombinations(
  all = all,
  parentList = biologicalHierarchy
)

## ----kontextual---------------------------------------------------------------
# Running Kontextual on all relationships across all images.
kerenKontextual <- Kontextual(
  cells = kerenSCE,
  parentDf = parentDf,
  r = 100,
  cores = nCores
)

## ----head kontextual results--------------------------------------------------
head(kerenKontextual, 10)

## ----survival data------------------------------------------------------------
# add survival column to kerenSCE
kerenSCE$event = 1 - kerenSCE$Censored
kerenSCE$survival = Surv(kerenSCE$Survival_days_capped, kerenSCE$event)

## ----prep kontextual matrix---------------------------------------------------
# Converting Kontextual result into data matrix
kontextMat <- prepMatrix(kerenKontextual)

# Ensuring rownames of kontextMat match up with the image IDs of the SCE object
kontextMat <- kontextMat[kerenSCE$imageID |> unique(), ]

# Replace NAs with 0
kontextMat[is.na(kontextMat)] <- 0

## ----survival anaylysis-------------------------------------------------------
# Running survival analysis
survivalResults = spicy(cells = kerenSCE,
                        alternateResult = kontextMat,
                        condition = "survival",
                        weights = TRUE)

head(survivalResults$survivalResults, 10)


## ----survival signifPlot------------------------------------------------------
signifPlot(survivalResults)

## ----extract data KM curve----------------------------------------------------
# Extracting survival data
survData <- kerenSCE |>
  colData() |>
  data.frame() |>
  select(imageID, survival) |>
  unique()

# Creating survival vector
kerenSurv <- survData$survival
names(kerenSurv) <- survData$imageID

kerenSurv

## ----KM curve, fig.width=5, fig.height=4--------------------------------------
# Selecting most significant relationship
survRelationship <- kontextMat[["DC__NK__immune"]]
survRelationship <- ifelse(survRelationship > median(survRelationship), "Localised", "Dispersed")

# Plotting Kaplan-Meier curve
survfit2(kerenSurv ~ survRelationship) |>
  ggsurvfit() +
  ggtitle("DC__NK__immune")

## ----calc metrics-------------------------------------------------------------
kerenSCE <- getDistances(kerenSCE,
  maxDist = 200,
  nCores = 1
)

kerenSCE <- getAbundances(kerenSCE,
  r = 200,
  nCores = 1
)

## ----calcStateChanges---------------------------------------------------------
stateChanges <- calcStateChanges(
  cells = kerenSCE,
  type = "distances",
  image = "6",
  from = "Keratin_Tumour",
  to = "Macrophages",
  marker = "p53",
  nCores = 1
)

stateChanges

## ----plotStateChanges p53-----------------------------------------------------
p <- plotStateChanges(
  cells = kerenSCE,
  type = "distances",
  image = "6",
  from = "Keratin_Tumour",
  to = "Macrophages",
  marker = "p53",
  size = 1,
  shape = 19,
  interactive = FALSE,
  plotModelFit = FALSE,
  method = "lm"
)

p

## ----calcStateChanges all-----------------------------------------------------
stateChanges <- calcStateChanges(
  cells = kerenSCE,
  type = "distances",
  nCores = 1,
  minCells = 100
)

stateChanges |>
  filter(imageID == 6) |>
  head(n = 10)

## ----plotStateChanges HLA-----------------------------------------------------
p <- plotStateChanges(
  cells = kerenSCE,
  type = "distances",
  image = "6",
  from = "Keratin_Tumour",
  to = "Macrophages",
  marker = "HLA_Class_1",
  size = 1,
  shape = 19,
  interactive = FALSE,
  plotModelFit = FALSE,
  method = "lm"
)

p

## ----statChanges--------------------------------------------------------------
stateChanges |> head(n = 10)

## ----plotStateChanges CD20----------------------------------------------------
p <- plotStateChanges(
  cells = kerenSCE,
  type = "distances",
  image = "35",
  from = "CD4_Cell",
  to = "B",
  marker = "CD20",
  size = 1,
  shape = 19,
  interactive = FALSE,
  plotModelFit = FALSE,
  method = "lm"
)

p

## ----calcContamination--------------------------------------------------------
kerenSCE <- calcContamination(kerenSCE)

stateChangesCorrected <- calcStateChanges(
  cells = kerenSCE,
  type = "distances",
  nCores = 1,
  minCells = 100,
  contamination = TRUE
)

stateChangesCorrected |> head(n = 20)

## ----corrected----------------------------------------------------------------
cellTypeMarkers <- c("CD3", "CD4", "CD8", "CD56", "CD11c", "CD68", "CD45", "CD20")

values <- c("blue", "red")
names(values) <- c("None", "Corrected")

df <- rbind(
  data.frame(TP = cumsum(stateChanges$marker %in% cellTypeMarkers), FP = cumsum(!stateChanges$marker %in% cellTypeMarkers), type = "None"),
  data.frame(TP = cumsum(stateChangesCorrected$marker %in% cellTypeMarkers), FP = cumsum(!stateChangesCorrected$marker %in% cellTypeMarkers), type = "Corrected")
)

ggplot(df, aes(x = TP, y = FP, colour = type)) +
  geom_line() +
  labs(y = "Cell state marker", x = "Cell type marker") +
  scale_colour_manual(values = values)

## ----corrected zoom-----------------------------------------------------------
ggplot(df, aes(x = TP, y = FP, colour = type)) +
  geom_line() +
  xlim(0, 100) +
  ylim(0, 1000) +
  labs(y = "Cell state marker", x = "Cell type marker") +
  scale_colour_manual(values = values)

## ----survival state changes---------------------------------------------------
# Preparing features for Statial
stateMat <- prepMatrix(stateChanges)

# Ensuring rownames of stateMat match up with rownames of the survival vector
stateMat <- stateMat[names(kerenSurv), ]

# Remove some very small values
stateMat <- stateMat[, colMeans(abs(stateMat) > 0.0001) > .8]

survivalResults <- colTest(stateMat, kerenSurv, type = "survival")

head(survivalResults)

## ----KM curve state changes, fig.width=5, fig.height=4------------------------
# Selecting the most significant relationship
survRelationship <- stateMat[["Keratin_Tumour__CD4_Cell__Keratin6"]]
survRelationship <- ifelse(survRelationship > median(survRelationship), "Higher expression in close cells", "Lower expression in close cells")

# Plotting Kaplan-Meier curve
survfit2(kerenSurv ~ survRelationship) |>
  ggsurvfit() +
  add_pvalue() +
  ggtitle("Keratin_Tumour__CD4_Cell__Keratin6")

## ----lisaClust----------------------------------------------------------------
set.seed(51773)

# Preparing features for lisaClust
kerenSCE <- lisaClust::lisaClust(kerenSCE, k = 5)

## ----hatchingPlot, fig.height=5, fig.width=6.5--------------------------------
# Use hatching to visualise regions and cell types.
lisaClust::hatchingPlot(kerenSCE,
  useImages = "5",
  line.spacing = 41, # spacing of lines
  nbp = 100 # smoothness of lines
)

## ----colTest regions----------------------------------------------------------
cellTypeRegionMeans <- getMarkerMeans(kerenSCE,
  imageID = "imageID",
  cellType = "cellType",
  region = "region"
)

survivalResults <- colTest(cellTypeRegionMeans[names(kerenSurv), ], kerenSurv, type = "survival")

head(survivalResults)

## ----ClassifyR----------------------------------------------------------------
# Calculate cell type and region proportions
cellTypeProp <- getProp(kerenSCE,
  feature = "cellType",
  imageID = "imageID"
)
regionProp <- getProp(kerenSCE,
  feature = "region",
  imageID = "imageID"
)

# Combine all the features into a list for classification
featureList <- list(
  states = stateMat,
  kontextual = kontextMat,
  regionMarkerMeans = cellTypeRegionMeans,
  cellTypeProp = cellTypeProp,
  regionProp = regionProp
)

# Ensure the rownames of the features match the order of the survival vector
featureList <- lapply(featureList, function(x) x[names(kerenSurv), ])


set.seed(51773)

kerenCV <- crossValidate(
  measurements = featureList,
  outcome = kerenSurv,
  classifier = "CoxPH",
  selectionMethod = "CoxPH",
  nFolds = 5,
  nFeatures = 10,
  nRepeats = 20,
  nCores = 1
)

## ----performancePlot----------------------------------------------------------
# Calculate AUC for each cross-validation repeat and plot.
performancePlot(kerenCV,
  characteristicsList = list(x = "Assay Name")
) +
  theme(axis.text.x = element_text(angle = 45, vjust = 1, hjust = 1))

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

#' Create all combinations of cell type relationships from a list of parents
#'
#' @description
#' This function takes in named vectors of all the parent populations in the
#' dataset, and creates a data frame containing all pairwise cell relationships,
#' this data frame can be inputed into the `parentDf` argument in `Kontextual`.
#'
#' @param all A list of all the `to` cell types Kontextual is evaluated over.
#' @param parentList a named list where the names correspond to parent names and
#' the values contain a vector of children for that parent. Note: If parentList
#' is specified the `...` argument will be ignored, see examples.
#' @param ... Vectors of each parent population.
#'
#' @return A data frame containing all pairwise cell relationships and their
#' corresponding parent
#'
#' @examples
#' # Example 1, using `parentList`
#'
#' parentList <- list(
#'   "tcells" = c("CD4", "CD8"),
#'   "tissue" = c("epithelial", "stromal")
#' )
#' 
#' allCells <- c("tumour", "CD4", "CD8", "epithelial", "stromal")
#' 
#' parentCombinations(all = allCells, parentList = parentList)
#' 
#' 
#' # Example 2, with `...` operator
#' tcells <- c("CD4", "CD8")
#' tissue <- c("epithelial", "stromal")
#' allCells <- c("tumour", tissue, tcells)
#'
#' parentCombinations(all = allCells, tcells, tissue)
#'
#' @export
#' @rdname parentCombinations
#' @import dplyr
#' @import tidyr
#'
parentCombinations <- function(all, ..., parentList = NULL) {
  
  if(is.null(parentList)) {
    # Gets variable names of all the parent vector
    names <- as.list(substitute(c(...)))[-1]
    
    parentList <- list(...)
    names(parentList) <- names
  }

  # Creates data.frame of parent name and parent vector
  parentTable <- data.frame(
    parent_name = names(parentList),
    parent = I(parentList)
  )


  # Creates all combination of parent and child
  parentDfs <- lapply(
    seq_along(parentList),
    function(x) {
      return(tidyr::crossing(
        to = parentList[[x]],
        parent_name = names(parentList)[x]
      ))
    }
  )

  parentDf <- bind_rows(parentDfs) |>
    merge(parentTable, by = "parent_name") |>
    expand_grid(from = unique(all)) |>
    data.frame() |>
    select("from", "to", "parent", "parent_name") |>
    filter(from != to)



  return(parentDf)
}




#' Extract parent and all children from a Phylo object
#'
#' @description
#' This function takes in a `phylo` object or a `treekoR` result from the 
#' \code{\link[treekoR]{getClusterTree}} function, and converts its into a 
#' named list of each and children to input into 
#' \code{\link[Statial]{parentCombinations}}. 
#' 
#' \bold{Note}: Parent populations with one child will be pruned. Make sure to 
#' include this cell type in the `all` vector when using 
#' \code{\link[Statial]{parentCombinations}} to ensure this cell type is included
#' in pairwise calculations.
#
#'
#' @param phylo_tree a phylo object or a treekoR result.
#'
#' @return A named list of parents and their respective children.
#'
#'
#' @export
#' @rdname getParentPhylo
#' @importFrom dplyr rename filter mutate group_by summarise
getParentPhylo = function(phylo_tree) {
  
  # Get tree from treekor object
  if(class(phylo_tree) == "list" && "clust_tree" %in% names(phylo_tree)) {
    phylo_tree = phylo_tree$clust_tree
  }
  
  if(!class(phylo_tree) == "phylo") {
    stop("Please input a phylo object or a treekoR result")
  }
  
  node_labels <- c(phylo_tree$tip.label)
  node_length = length(node_labels)
  names(node_labels) = 1:node_length
  
  edge_matrix <- data.frame(phylo_tree$edge) |> 
    rename("parent" = "X1",
           "child" = "X2") |> 
    filter(child <= node_length) |> 
    mutate(child = node_labels[child]) |>
    mutate(child = unname(child)) |> 
    group_by(parent) |> 
    summarise(children = list(child)) |> 
    filter(lengths(children) > 1)
  
  child_list = edge_matrix$children
  
  names(child_list) = paste("parent", seq_along(child_list), sep = "_")
  
  return(child_list)
}

#' Evaluation of pairwise cell relationships, conditional on a 3rd population.
#'
#' @description
#' Kontextual identifies the relationship between two cell types which are
#' conditional on the spatial behaviour of a 3rd cell population, for a
#' particular radius (r).
#'
#'
#' @param cells A SingleCellExperiment, SpatialExperiment or a list of
#' data.frames containing columns specifying the imageID, cellType, and x and y
#' spatial coordinates.
#' @param parentDf A data frame from \code{\link[Statial]{parentCombinations}}
#' @param r Radii to evaluated pairwise relationships between from and to cells.
#' @param from The first cell type to be evaluated in the pairwise relationship.
#' @param to The second cell type to be evaluated in the pairwise relationship.
#' @param parent The parent population of the from cell type (must include from cell type).
#' @param image A vector of images to subset the results to. If NULL we default to all images.
#' @param inhom  A logical value indicating whether to account for inhomogeneity.
#' @param edgeCorrect A logical value indicating whether to perform edge correction.
#' @param window Type of window for data, either `square`, `convex` or `concave`,
#'  passed into \code{\link[Statial]{makeWindow}}
#' @param window.length A tuning parameter for controlling the level of concavity when estimating concave windows.
#' Passed into \code{\link[Statial]{makeWindow}}
#' @param includeOriginal A logical value to return the original L function
#' values along with the kontextual values.
#' @param spatialCoords The columns which contain the x and y spatial coordinates.
#' @param cellType The column which contains the cell types.
#' @param imageID The column which contains image identifiers.
#' @param cores Number of cores for parallel processing.
#' @return A kontextualResult object
#'
#' @examples
#' # Load data
#' data("kerenSCE")
#'
#'
#' CD4_Kontextual <- Kontextual(
#'   cells = kerenSCE,
#'   r = 50,
#'   from = "Macrophages",
#'   to = "Keratin_Tumour",
#'   parent = c("Macrophages", "CD4_Cell"),
#'   image = "6"
#' )
#'
#'
#' head(CD4_Kontextual)
#'
#' @export Kontextual
#' @rdname Kontextual
#' @importFrom BiocParallel bpmapply
#' @importFrom SummarizedExperiment colData
#' @importFrom tibble remove_rownames
#' @importFrom methods is
#' @importFrom stats runif
#' @importFrom data.table as.data.table 

Kontextual <- function(cells,
                       r,
                       parentDf = NULL,
                       from = NULL,
                       to = NULL,
                       parent = NULL,
                       image = NULL,
                       inhom = FALSE,
                       edgeCorrect = TRUE,
                       window = "convex",
                       window.length = NA,
                       includeOriginal = TRUE,
                       spatialCoords = c("x", "y"),
                       cellType = "cellType",
                       imageID = "imageID",
                       cores = 1) {
  if (is.null(parentDf) &
    is.null(from) &
    is.null(to) &
    is.null(parent)) {
    stop("Please specificy a parentDf (obtained from parentCombinations), or from, to, and parent cellTypes")
  } else if (!is.null(from) &
    !is.null(to) &
    !is.null(parent)
  ) {
    parentDf <- data.frame(
      from = from,
      to = to,
      parent = I(list(parent))
    )
  }

  if (is(cells, "list")) {
    cells <- lapply(
      cells,
      validateDf,
      imageID = imageID,
      cellType = cellType,
      spatialCoords = spatialCoords,
      image = image
    )
  }
  
  if (is(cells, "SpatialExperiment")) {
    cd <- cbind(colData(cells), SpatialExperiment::spatialCoords(cells)) |>
      data.frame()
    if(!all(spatialCoords%in%colnames(cd))) spatialCoords <- colnames(SpatialExperiment::spatialCoords(cells))
    cells <- cd
    }

  if (is(cells, "SingleCellExperiment")) {
    cells <- cells |>
      SummarizedExperiment::colData() |>
      data.frame()
  }


  if (is(cells, "data.frame")) {
    cells <- validateDf(
      cells,
      imageID = imageID,
      cellType = cellType,
      spatialCoords = spatialCoords,
      image = image
    )

    # cells <- mutate(cells, imageID = as.character(imageID))
    cells <- split(cells, cells$imageID)
  }

  if (!is(cells, "list")) {
    stop("Cells must be one of the following: SingleCellExperiment, SpatialExperiment, or a list of data.frames with imageID, cellType, and x and y columns")
  }
  
  
  
  # Specify cores for parrellel computing
  x <- runif(1) # nolint
  BPPARAM <- Statial:::.generateBPParam(cores = cores)
  
  r = sort(r)

  cells <- lapply(cells, function(image) {
    image$cellID <- factor(seq_len(nrow(image)))
    return(image)
  })

  images <- cells
  
  # Convert images to PPP objects
  imagesPPP <- bplapply(images, function(image){
    # make window
    ow <- Statial::makeWindow(image, window, window.length)
    
    # Define marks
    marks = data.frame(
      cellType = image$cellType,
      cellID = image$cellID
    )
    
    # Make PPP object
    imagePPP <- spatstat.geom::ppp(
      x = image$x,
      y = image$y,
      window = ow,
      marks = marks
    )
    
    return(imagePPP)
  }, BPPARAM = BPPARAM ) 
  

  # Calculate Areas
  Areas = bplapply(imagesPPP, function(image) {
    return(spatstat.geom::area(image))
  }, BPPARAM = BPPARAM )


  
  imagesInfo <- data.frame(
    imageID = names(images),
    images = I(images),
    imagesPPP = I(imagesPPP),
    area = I(Areas)
  )

  imagesInfoR <- expand_grid(
    imagesInfo,
    r = r) |> 
    mutate(id = paste(imageID, r, sep = "."))
  
  # Calculate close pairs for all images and radii
  closePairList <- bplapply(imagesPPP, function(imagePPP) {

    # Calculate close pairs
    closePairs <- spatstat.geom::closepairs(
      imagePPP, max(r, na.rm = TRUE),
      what = "ijd", distinct = FALSE
    ) |>
      data.frame()

    cellTypes <- imagePPP$marks$cellType
    names(cellTypes) <- imagePPP$marks$cellID
    closePairs$cellTypeI <- cellTypes[(closePairs$i)]
    closePairs$cellTypeJ <- cellTypes[(closePairs$j)]
    closePairs$i <- factor(closePairs$i, levels = imagePPP$marks$cellID)
    
    closePairs <- as.data.table(closePairs)
   
    # Subsetting close pairs if there is more R values.
    if(length(r) > 1){
      closePairList = lapply(r[-length(r)], function(r){
        return(closePairs[d < r])
      })
      
      closePairList = append(closePairList, list(closePairs))
      names(closePairList) = r
    } else{
      closePairList = list(closePairs)
      names(closePairList) = r
    }
    
    return(closePairList)
    
  }, BPPARAM = BPPARAM)
  
  
  # Adding closePairs to the image data.frame
  closePairList = unlist(closePairList, recursive = FALSE)
  closePairsDfs = data.frame(id = names(closePairList), 
                             closePairs = I(closePairList))
  imagesInfoR = left_join(imagesInfoR,
                          closePairsDfs,
                          by = join_by(id))
  

  
  # Perform edge correction (if needed)
  imagesInfoR$closePairs <- bpmapply(function(imagePPP, closePairs, r){
    
    if(edgeCorrect){
      edge <- .borderEdge(imagePPP, r)
      edge <- as.data.frame(edge)
      edge$i <- factor(imagePPP$marks$cellID, levels = imagePPP$marks$cellID)
      edge$edge <- 1 / edge$edge
    } else {
      edge = data.frame(
        i = factor(imagePPP$marks$cellID, levels = imagePPP$marks$cellID),
        edge = 1
      )
    }

    closePairs <- left_join(closePairs, edge[, c("i", "edge")], by = "i")
  }, 
  imagePPP = imagesInfoR$imagesPPP,
  closePairs = imagesInfoR$closePairs,
  r = imagesInfoR$r,
  SIMPLIFY = FALSE,
  BPPARAM = BPPARAM)


  # Create all combinations of specified parameters
  allCombinations <- expand_grid(
    parentDf,
    inhomL = inhom
  )

  # Create data frame for mapply
  kontextualDf <- merge(imagesInfoR, allCombinations, all = TRUE) |>
    mutate("test" = paste(from, "__", to, sep = ""))

  if ("parent_name" %in% names(kontextualDf)) {
    kontextualDf <- mutate(
      kontextualDf,
      "test" = paste(test, "__", parent_name, sep = "")
    )
  }

  # Calculate Kontextual values
  lVals <- bpmapply(
    KontextualCore,
    images = kontextualDf$images,
    r = kontextualDf$r,
    closePairs = kontextualDf$closePairs,
    from = kontextualDf$from,
    to = kontextualDf$to,
    parent = kontextualDf$parent,
    inhom = kontextualDf$inhomL,
    area = kontextualDf$area,
    SIMPLIFY = FALSE,
    BPPARAM = BPPARAM
  )
  

  # Combine data.frame rows
  lVals <- lVals |>
    bind_rows()

  lValsClean <- kontextualDf |>
    mutate("parent_name" = "") |>
    select(-c("images", "from", "to", "parent_name", "parent")) |>
    cbind(lVals) |>
    remove_rownames()

  if (includeOriginal == FALSE) {
    lValsClean <- lValsClean |>
      select(
        "imageID",
        "test",
        "kontextual",
        "r"
      )
  } else {
    lValsClean <- lValsClean |>
      select(
        "imageID",
        "test",
        "original",
        "kontextual",
        "r",
        "inhomL"
      )
  }

  return(lValsClean)
}




#' @noRd
#'
#' @importFrom data.table dcast
KontextualCore <- function(images,
                           r,
                           from,
                           to,
                           parent,
                           closePairs,
                           area,
                           inhom = FALSE,
                           edge = FALSE,
                           includeOriginal = TRUE,
                           returnWeight = FALSE,
                           ...) {
  
  # Returns NA if to and from cell types not in image
  if (!(c(to, from) %in% unique(images$cellType) |> all())) {
    condL <- data.frame(original = NA, kontextual = NA)
    rownames(condL) <- paste(from, "__", to)
    return(condL)
  }
  
  
  child1 = from
  child2 = to
  
  # Convert closepairs to counts of cells next to other cells
  counts <- closePairs[, .(n = sum(edge)), by = .(i, cellTypeI, cellTypeJ)]
  counts <- dcast(counts, i + cellTypeI ~ cellTypeJ, value.var = "n", fill = 0)

  
  ########
  # Calculate statistics
  ########
  
  Kontextual <- .Kontext(closePairs, counts, child1, child2, parent, r, returnWeight)
  
  if (inhom) {
    L <- .Linhomfunction(closePairs, counts, child1, child2, r, area)
  } else {
    L <- .Lfunction(closePairs, counts, child1, child2, r, area)
  }
  
  if (returnWeight) {
    return(Kontext)
  }
  
  
  # return data frame of original and kontextual values.
  condL <- data.frame(
    original = L,
    kontextual = Kontextual
  )

  rownames(condL) <- paste(from, "__", to)


  return(condL)
}


#' @noRd
#'
#' @importFrom dplyr rename
validateDf <- function(cells, cellType, imageID, spatialCoords, image = NULL) {
  result <- try(
    {
      cells <- cells[, c(cellType, imageID, spatialCoords)] |>
        rename(
          "cellType" = cellType,
          "imageID" = imageID,
          "x" = spatialCoords[1],
          "y" = spatialCoords[2]
        )
    },
    silent = TRUE
  )

  if (is(result, "try-error")) {
    stop("Please specifiy imageID or cellType or spatialCoords")
  }

  if (!is.null(image)) cells <- cells[cells$imageID %in% image, ]

  return(cells)
}




#' Test whether an object is a kontextualResult
#'
#' @param kontextualResult a object to test
#'
#' @examples
#' data <- data.frame()
#' if (!isKontextual(data)) print("Not a kontextualResult")
#'
#' @export isKontextual
#' @rdname isKontextual
isKontextual <- function(kontextualResult) {
  colNames <- c(
    "imageID",
    "test",
    "kontextual",
    "r"
  )

  return(all(colNames %in% names(kontextualResult)))
}

#' Convert Kontextual or state changes result to a matrix for classification
#'
#' @param result a kontextual or state changes result data.frame.
#' @param replaceVal value which NAs are replaced with.
#' @param column The column which contains the scores that you want to select.
#' @param test A column containing which will be the column names of the expanded matrix.
#'
#' @examples
#' data("kerenSCE")
#'
#'
#' CD4_Kontextual <- Kontextual(
#'   cells = kerenSCE,
#'   r = 50,
#'   from = "Macrophages",
#'   to = "Keratin_Tumour",
#'   parent = c("Macrophages", "CD4_Cell"),
#'   image = "6"
#' )
#'
#'
#' kontextMat <- prepMatrix(CD4_Kontextual)
#'
#' @export prepMatrix
#' @rdname prepMatrix
#' @importFrom dplyr mutate
prepMatrix <- function(result,
                       replaceVal = 0,
                       column = NULL,
                       test = NULL) {
  mat <- NULL

  if ("kontextual" %in% colnames(result)) {
    mat <- result |>
      # Implement support for multiple values in other columns.
      dplyr::select(imageID, test, kontextual) |>
      tidyr::pivot_wider(names_from = test, values_from = kontextual, values_fill = replaceVal) |>
      tibble::column_to_rownames("imageID") 
    
    attr(mat, "kontextualResult") = TRUE
  }

  if ("primaryCellType" %in% colnames(result)) {
    if (!is.null(column)) {
      result$type <- result[, column]
    } else {
      result$type <- result$coef
    }

    mat <- result |>
      as.data.frame() |>
      # Implement support for multiple values in other columns.
      dplyr::mutate(
        test = paste(primaryCellType, otherCellType, marker, sep = "__")
      ) |>
      dplyr::select(imageID, test, type) |>
      tidyr::pivot_wider(
        names_from = test, values_from = type, values_fill = replaceVal
      ) |>
      tibble::column_to_rownames("imageID")
  }



  mat
}

test_that("Expecting an error when parentDf or to, from, parent is NULL", {
  data("kerenSCE")

  testthat::expect_error(
    Kontextual(
      cells = kerenSCE,
      r = 50,
      parentDf = NULL,
      to = NULL,
      from = NULL,
      parent = NULL
    )
  )
})


# test_that("Kontextual values from kerenSCE are the same as the saved ones", {
#   data("kerenSCE")
#
#   expected_data <- readRDS("CD4_Kontextual.rds")
#
#   CD4_Kontextual <-
#     Kontextual(
#       cells = kerenSCE,
#       r = 50,
#       from = "TC_CD4",
#       to = "SC5",
#       parent = c("TC_CD4", "TC_CD8"),
#       inhom = TRUE,
#       edgeCorrect = FALSE,
#       window = "convex",
#       window.length = NA,
#       weightQuantile = .80,
#       includeZeroCells = TRUE,
#       includeOriginal = TRUE
#     )
#
#   testthat::expect_equal(
#     CD4_Kontextual,
#     expected_data
#   )
# })


test_that("Expecting an error when dataset contains wrong column names", {
  data("kerenSCE")

  renamedSCE <- colData(kerenSCE) |>
    data.frame() |>
    rename("imageName" = "imageID")

  testthat::expect_error(
    suppressWarnings(Kontextual(
      cells = renamedSCE,
      r = 50,
      from = "TC_CD4",
      to = "SC5",
      parent = c("TC_CD4", "TC_CD8")
    ))
  )
})


test_that("Fail on invalid input", {
  wrong_input <- c("wrong", "input")

  testthat::expect_error(
    Kontextual(
      cells = wrong_input,
      r = 50,
      from = "TC_CD4",
      to = "SC5",
      parent = c("TC_CD4", "TC_CD8")
    )
  )
})

#' @noRd
#'
#' @importFrom BiocParallel SerialParam SnowParam MulticoreParam bpparam
.generateBPParam <- function(cores = 1) {
  seed <- .Random.seed[1]

  if (cores == 1) {
    BPparam <- BiocParallel::SerialParam(RNGseed = seed)
  } else { ## Parallel processing is desired.
    ## Also set the BPparam RNGseed if the user ran set.seed(someNumber) themselves.
    if (Sys.info()["sysname"] == "Windows") { # Only SnowParam suits Windows.
      BPparam <- BiocParallel::SnowParam(
        min(
          cores,
          BiocParallel::snowWorkers("SOCK")
        ),
        RNGseed = seed
      )
    } else if (Sys.info()["sysname"] %in% c("MacOS", "Linux")) {
      BPparam <- BiocParallel::MulticoreParam(
        min(
          cores,
          BiocParallel::multicoreWorkers()
        ),
        RNGseed = seed
      )
      ## Multicore is faster than SNOW, but it doesn't work on Windows.
    } else { ## Something weird.
      BPparam <- BiocParallel::bpparam() ## BiocParallel will figure it out.
    }
  }

  BPparam
}

#' Cell permutation for Kontextual
#'
#' @description
#' Function which randomises specified cells in an image and calculates
#' the `Kontextual` value. This can be used to estimate the null distribution,
#' of the parent cell population for significance testing.
#'
#' @param cells A single image data frame from a SingleCellExperiment object
#' @param nSim Number of randomisations which will be calculated.
#' @param r Radius to evaluated pairwise relationships between from and to cells.
#' @param from The first cell type to be evaluated in the pairwise relationship.
#' @param to The second cell type to be evaluated in the pairwise relationship.
#' @param parent The parent population of the from cell type (must include from cell type).
#' @param image A vector of images to subset the results to. If NULL we default to all images.
#' @param returnImages A logical value to indicate whether the function should
#' return the randomised images along with the Kontextual values.
#' @param inhom A logical value indicating whether to account for inhomogeneity.
#' @param edge A logical value indicating whether to perform edge correction.
#' @param cores Number of cores for parallel processing.
#' @param spatialCoords A character vector containing the names of the two
#'     spatial dimansions in the data. Defaults to `c("x", "y")`.
#' @param cellType The name of the cell type field in the data. Defualts to
#'     "cellType".
#' @param imageID The name of the image ID field in the data. Defualts to
#'     "imageID".
#' @param ... Any arguments passed into \code{\link[Statial]{Kontextual}}
#' @return A data frame containing Kontextual value for each randomised image.
#' If `returnImages = TRUE` function will return a list with Kontextual values
#' and the randomised images.
#'
#' @examples
#' data("kerenSCE")
#'
#' kerenImage6 <- kerenSCE[, kerenSCE$imageID == "6"]
#'
#' relabelResult <- relabelKontextual(
#'   cells = kerenImage6,
#'   nSim = 5,
#'   r = 250,
#'   from = "CD4_Cell",
#'   to = "Keratin_Tumour",
#'   parent = c("CD4_Cell", "Macrophages"),
#'   cores = 2
#' )
#'
#' @export
#' @rdname relabelKontextual
#' @importFrom dplyr mutate
#' @importFrom BiocParallel  bplapply
relabelKontextual <- function(cells,
                              nSim = 1,
                              r,
                              from,
                              to,
                              parent,
                              image = NULL,
                              returnImages = FALSE,
                              inhom = TRUE,
                              edge = FALSE,
                              cores = 1,
                              spatialCoords = c("x", "y"),
                              cellType = "cellType",
                              imageID = "imageID",
                              ...) {
  if (is(cells, "SingleCellExperiment")) {
    cells <- cells |>
      SingleCellExperiment::colData() |>
      data.frame()
  }

  if (is(cells, "SpatialExperiment")) {
    cells <- cbind(colData(cells), spatialCoords(cells)) |>
      data.frame()
  }

  if (is(cells, "data.frame")) {
    cells <- validateDf(
      cells,
      imageID = imageID,
      cellType = cellType,
      spatialCoords = spatialCoords,
      image = image
    )
  }

  if (!is(cells, "data.frame")) {
    stop("Cells must be one of the following: SingleCellExperiment, SpatialExperiment, or a list of data.frames with imageID, cellType, and x and y columns")
  }

  imageArray <- replicate(nSim, cells, simplify = FALSE)


  # relabel cells in parent population for all images in imageArray
  relabeled <- bplapply(imageArray,
    relabel,
    labels = parent,
    BPPARAM = MulticoreParam(workers = cores)
  )

  relabeled <- c(list(cells), relabeled)
  names(relabeled) <- as.character(seq_along(relabeled))

  parentDf <- data.frame(
    from = from,
    to = to,
    parent = I(list(parent))
  )

  # Calculated the child and parent values for the relabeled images
  relabeledDf <- Kontextual(
    cells = relabeled,
    r = r,
    parentDf = parentDf,
    inhom = inhom,
    edgeCorrect = edge,
    cores = cores,
    ...
  )

  relabeledDf <- relabeledDf |>
    select("imageID", "original", "kontextual", "r") |>
    mutate(type = ifelse(imageID == 1, "original", "randomised"))

  if (returnImages) {
    return(list(relabeledDf, relabeled))
  }

  return(relabeledDf)
}



#' Permute all specified cells labels in a single image
#'
#' @description
#' This function relabels all specified cells within a single image, to
#' estimate the null distribution of cell population specified.
#'
#' @param image A single image from a Single Cell Experiment object.
#' @param labels A vector of CellTypes labels to be permuted If NULL all cells
#' labels will be radomised.
#'
#' @return A data frame containing all pairwise cell relationships and their
#' corresponding parent
#'
#' @examples
#' data("kerenSCE")
#'
#' kerenImage6 <- kerenSCE[, kerenSCE$imageID == "6"]
#'
#' kerenImage6 <- kerenImage6 |>
#'   SingleCellExperiment::colData() |>
#'   data.frame()
#'
#' # Permute CD8 T cells and T cell labels in the image
#' relabeledImage <- relabel(kerenImage6, labels = c("p53", "Keratin+Tumour"))
#' plot(relabeledImage)
#'
#' @export
#' @rdname relabelKontextual
#' @import dplyr
#'
relabel <- function(image, labels = NULL) {
  # if labels are NULL relabel the whole image, otherwise relabel just the specified marks
  if (is.null(labels)) {
    relabeledCells <- image |> mutate(cellType = sample(cellType))
  } else {
    # split up cells into subset which will be relabeled, and subset which wont be relabeled
    notRelabel <- image |> filter(!(cellType %in% labels))
    toRelabel <- image |> filter(cellType %in% labels)

    # relabel toRelabel cells
    relabeled <- toRelabel |> mutate(cellType = sample(cellType))

    # combine relabeled cells and non relabeled cells
    relabeledCells <- rbind(relabeled, notRelabel)
  }

  return(relabeledCells)
}

#' Evaluation of Kontextual over a range of radii.
#'
#' @description
#' This function obtains `Kontextual` values over a range of radii, standard
#' deviations for each value can be obtained using permutation for significance
#' testing. To obtain estimates for standard deviations specify `se = TRUE`.
#'
#'
#' @param cells A single image from a SingleCellExperiment object
#' @param from The first cell type to be evaluated in the pairwise relationship.
#' @param to The second cell type to be evaluated in the pairwise relationship.
#' @param parent The parent population of the from cell type (must include from cell type).
#' @param image A vector of images to subset the results to. If NULL we default to all images.
#' @param rs A vector of radii to evaluate kontextual over.
#' @param inhom A logical value indicating whether to perform an inhomogeneous L function.
#' @param edge A logical value indicating whether to perform edge correction.
#' @param se A logical value to indicate if the standard deviation of
#'  kontextual should be calculated to construct error bars.
#' @param nSim Number of randomisations to perform using \code{\link[Statial]{relabelKontextual}},
#'  which will be used to calculated the SE.
#' @param cores Number of cores for parallel processing.
#' @param imageID The column in colData that stores the image ids.
#' @param cellType The column in colData that stores the cell types.
#' @param ... Any arguments passed into \code{\link[Statial]{Kontextual}}.
#'
#' @return A data frame of original L values and Kontextual values evaluated
#' over a range of radii.
#'
#' @examples
#'
#' data("kerenSCE")
#'
#' kerenImage6 <- kerenSCE[, kerenSCE$imageID == "6"]
#'
#' rsDf <- kontextCurve(
#'   cells = kerenSCE,
#'   from = "CD4_Cell",
#'   to = "Keratin_Tumour",
#'   parent = c("CD4_Cell", "Macrophages"),
#'   rs = seq(10, 510, 100),
#'   cores = 2
#' )
#'
#' @export
#' @rdname kontextCurve
#' @importFrom stats sd
#' @importFrom dplyr filter select group_by summarise
kontextCurve <- function(cells,
                         from,
                         to,
                         parent,
                         image = NULL,
                         rs = seq(10, 100, 10),
                         inhom = FALSE,
                         edge = TRUE,
                         se = FALSE,
                         nSim = 20,
                         cores = 1,
                         imageID = "imageID",
                         cellType = "cellType",
                         ...) {
  kontextualVals <- Kontextual(
    cells = cells,
    from = from,
    to = to,
    parent = parent,
    image = image,
    r = rs,
    inhom = inhom,
    edgeCorrect = edge,
    cores = cores,
    includeOriginal = TRUE,
    imageID = imageID,
    cellType = cellType,
    ...
  )

  rsDf <- kontextualVals |>
    dplyr::select("r", "original", "kontextual")


  if (se == TRUE) {
    seDf <- relabelKontextual(
      cells = cells,
      nSim = nSim,
      r = rs,
      image = image,
      from = from,
      to = to,
      parent = parent,
      returnImages = FALSE,
      inhom = inhom,
      edge = edge,
      cores = cores,
      ...
    )

    seDf <- seDf |>
      dplyr::filter(type != "original") |>
      dplyr::select("r", "original", "kontextual") |>
      dplyr::group_by(r) |>
      dplyr::summarise(
        "originalSd" = sd(original),
        "kontextualSd" = sd(kontextual)
      )

    rsDf <- merge(rsDf, seDf, by = "r")
  }

  return(rsDf)
}



#' Plotting the original and kontextual L values over a range of radii.
#'
#' @description
#' This function takes outputs from rsCurve and plots
#' them in ggplot. If standard deviation is estimated in rsCurve,
#' then confidence intervals will be constructed based on the standard deviation.
#' If the confidence interval overlaps with 0, then the relationship is insignificant
#' for that radius.
#'
#' @param rsDf A data frame from \code{\link[Statial]{kontextCurve}}.
#'
#' @return A ggplotly object showing the original and kontextual L function
#'  values over a range of radii
#'
#' @examples
#' data("kerenSCE")
#'
#' kerenImage6 <- kerenSCE[, kerenSCE$imageID == "6"]
#'
#' rsDf <- kontextCurve(
#'   cells = kerenImage6,
#'   from = "p53",
#'   to = "Immune",
#'   parent = c("p53", "Keratin+Tumour"),
#'   rs = seq(10, 510, 100),
#'   cores = 2
#' )
#'
#' kontextPlot(rsDf)
#'
#' @export
#' @rdname kontextPlot
#' @importFrom ggplot2 ggplot geom_point geom_hline geom_line geom_ribbon geom_smooth
#'  labs guides
#' @importFrom stringr str_detect
#' @importFrom dplyr select
#' @importFrom tidyselect starts_with
kontextPlot <- function(rsDf) {
  if (str_detect(names(rsDf), "Sd") |> any()) {
    kontextual <- rsDf |>
      select(r, starts_with("kontextual")) |>
      mutate(
        "lower" = kontextual - kontextualSd,
        "upper" = kontextual + kontextualSd
      ) |>
      select(-"kontextualSd") |>
      pivot_longer(kontextual)


    original <- rsDf |>
      select("r", starts_with("original")) |>
      mutate(
        "lower" = original - originalSd,
        "upper" = original + originalSd
      ) |>
      select(-"originalSd") |>
      pivot_longer(original)

    seDf <- rbind(kontextual, original)


    p <- ggplot(seDf, aes(x = r, y = value, col = name)) +
      geom_point() +
      geom_ribbon(aes(ymin = lower, ymax = upper, fill = name, alpha = 0.2)) +
      geom_hline(
        yintercept = 0,
        linetype = "dashed",
        color = "red"
      ) +
      geom_smooth(formula = y ~ x, method = "loess", se = FALSE) +
      guides(alpha = "none") +
      labs(
        x = "Radius (r)",
        y = "Relationship value",
        fill = "Function",
        col = "Function"
      )
  } else {
    p <- rsDf |>
      pivot_longer(-r) |>
      ggplot(aes(x = r, y = value, col = name)) +
      geom_point() +
      geom_hline(yintercept = 0, linetype = "dashed", color = "red") +
      geom_smooth(formula = y ~ x, method = "loess", se = FALSE) +
      labs(
        x = "Radius (r)",
        y = "Relationship value",
        fill = "Function",
        col = "Function"
      )
  }

  return(p)
}

#' MIBI-TOF Breast cancer intensities
#'
#' @description
#' This is a single MIBI-TOF data of breast cancer from patient 6 of the
#' Keren et al 2018 dataset.
#'
#'
#' @usage data(kerenSCE)
#'
#' @format kerenSCE a SingleCellExperiment object
#' @aliases
#' kerenSCE
#'
#' @references Keren, L., Bosse, M., Marquez, D., Angoshtari, R., Jain,
#' S., Varma, S., Yang, S. R., Kurian, A., Van Valen, D., West, R., Bendall,
#' S. C., & Angelo, M. (2018). A Structured Tumor-Immune Microenvironment in
#' Triple Negative Breast Cancer Revealed by Multiplexed Ion Beam Imaging.
#' Cell, 174(6), 1373-1387.e1319. ([DOI](https://doi.org/10.1016/j.cell.2018.08.039))
#'
#'
"kerenSCE"


#' Kontextual results from kerenSCE
#'
#' @description
#' This is a kontextual results data.frame created using Kontextual on the kerenSCE dataset.
#'
#'
#' @usage data(kerenKontextual)
#'
#' @format kerenKontextual a kontextual results object.
#' @aliases
#' kerenKontextual
#'
#'
#'
"kerenKontextual"

#' Creates a window for a PPP object
#'
#' @description
#' This function creates a window for a `spatstat::ppp` object, the type of
#' window can be specified using the `window` argument.
#'
#' @param data A single image data frame from a SingleCellExperiment object or PPP object.
#' @param window The shape of window around the regions, can be `square`, `convex` or `concave`
#' @param window.length A tuning parameter for controlling the level of concavity when estimating concave windows.
#' @return Creates an `owin` class, representing the observation window for the image.
#'
#' @examples
#' data <- data.frame(x = rnorm(10), y = rnorm(10))
#' ow <- makeWindow(data, window = "square")
#'
#' spatstat.geom::ppp(x = data$x, y = data$y, window = ow)
#'
#' @export
#' @rdname makeWindow
#' @import concaveman

makeWindow <- function(data,
                       window = "square",
                       window.length = NULL) {
  ow <-
    spatstat.geom::owin(xrange = range(data$x), yrange = range(data$y))


  if (window == "convex") {
    p <- spatstat.geom::ppp(data$x, data$y, ow)
    ow <- spatstat.geom::convexhull(p)
  }
  if (window == "concave") {
    message(
      "Concave windows are temperamental. Try choosing values of window.length > and < 1 if you have problems."
    )
    if (is.null(window.length) | is.na(window.length)) {
      window.length <- (max(data$x) - min(data$x)) / 20
    } else {
      window.length <- (max(data$x) - min(data$x)) / 20 * window.length
    }
    dist <- (max(data$x) - min(data$x)) / (length(data$x))
    bigDat <-
      do.call("rbind", lapply(as.list(as.data.frame(t(data[, c("x", "y")]))), function(x) {
        cbind(
          x[1] + c(0, 1, 0, -1, -1, 0, 1, -1, 1) * dist,
          x[2] + c(0, 1, 1, 1, -1, -1, -1, 0, 0) * dist
        )
      }))
    ch <-
      concaveman::concaveman(bigDat,
        length_threshold = window.length,
        concavity = 1
      )
    poly <- as.data.frame(ch[nrow(ch):1, ])
    colnames(poly) <- c("x", "y")
    ow <-
      spatstat.geom::owin(
        xrange = range(poly$x),
        yrange = range(poly$y),
        poly = poly
      )
  }
  ow
}

#' @noRd
#'
#' @importFrom magrittr %>%
#' @importFrom SummarizedExperiment assay
#' @importFrom SummarizedExperiment colData
#' @importFrom SummarizedExperiment colData<-
preProcessing <- function(SCE, intensities) {
  # Check if cellID exists in image already - if duplicated, reassign to
  # ImageCellID
  if (any(c("cellID", "CellID") %in% colnames(colData(SCE)))) {
    if (any(duplicated(colData(SCE)$cellID))) {
      colnames(colData(SCE))[which(names(colData(SCE)) == "cellID")] <- "imageCellID"
    }
    # TODO:Need to change CellID to cellID
    if (any(duplicated(colData(SCE)$CellID))) {
      colnames(colData(SCE))[which(names(colData(SCE)) == "CellID")] <- "ImageCellID"
    } else {
      colnames(colData(SCE))[which(names(colData(SCE)) == "CellID")] <- "cellID"
    }
  }

  if (!c("cellID") %in% colnames(colData(SCE))) {
    colData(SCE)$cellID <- rownames(colData(SCE))
  }


  intensitiesData <- data.frame(t(assay(SCE, intensities)))
  # spatialData <- data.frame(colData(SCE))

  # singleCellData <- cbind(spatialData[rownames(intensitiesData), ], intensitiesData)
  intensitiesData <- intensitiesData %>%
    mutate_all(~ ifelse(is.na(.), 0, .)) %>% # replace NAs with 0
    mutate_if(is.factor, as.character)
  intensitiesData <- intensitiesData[match(rownames(colData(SCE)), rownames(intensitiesData)), ]
  if (any(is.na(intensitiesData))) {
    stop("Number of cells in Intensities assay does not match number of cells in SCE")
  }
  colData(SCE) <- cbind(colData(SCE), intensitiesData)

  # Identify factor columns

  factor_cols <- which(sapply(colData(SCE), is.factor))


  for (x in factor_cols) {
    colData(SCE)[, x] <- colData(SCE)[, x] |> as.character()
  }

  return(SCE)
}


#' Calculate pairwise distance between cell types
#'
#' Calculates the euclidean distance from each cell to the nearest cell of each
#' type for a single image
#'
#' @param data the single cell data of interest
#' @param maxDist Maximum distance between pairs of points to be counted as close
#'   pairs.
#' @param distFun How to merge duplicate entries.
#'
#' @rdname distanceCalculator
#' @importFrom tibble rownames_to_column
#' @importFrom dplyr
#'   select mutate inner_join arrange group_by slice rename relocate full_join
#' @importFrom spatstat.geom owin ppp closepairs
#' @importFrom tidyr pivot_wider
distanceCalculator <- function(data, maxDist = 200, distFun = "min") {
  ow <- spatstat.geom::owin(
    xrange = range(data$x),
    yrange = range(data$y)
  )
  pppData <- spatstat.geom::ppp(
    x = data$x,
    y = data$y,
    window = ow,
    marks = data$cellType
  )

  closePairData <- spatstat.geom::closepairs(pppData, rmax = maxDist, what = "ijd")
  distanceData <- data.frame(
    cellID = data$cellID[closePairData$i],
    cellType = data$cellType[closePairData$j],
    d = closePairData$d
  )

  if (distFun == "min") {
    values_fn <- function(x) {
      min(c(x, maxDist), na.rm = TRUE)
    }
  }
  if (distFun == "abundance") {
    values_fn <- function(x) {
      sum(c(x, 0) > 0, na.rm = TRUE)
    }
  }

  values_fill <- values_fn(NULL)

  distanceData <- tidyr::pivot_wider(distanceData,
    names_from = cellType, values_from = d,
    values_fn = values_fn, values_fill = values_fill
  )
  distanceData <- dplyr::left_join(data[, "cellID", drop = FALSE], distanceData, by = "cellID") |>
    tibble::column_to_rownames("cellID")

  # distanceData[is.na(distanceData)] <- maxDist
  distanceData
}


#' Wrapper to calculate pairwise distance between cell types by image
#'
#' Calculates the euclidean distance from each cell to the nearest cell of each
#' type
#'
#' @param cells
#'   A dataframe with a cellType column as well as x and y spatial coordinates.
#'   The dataframe must contain a imageID column and cellID (unique cell
#'   identifier's) column as well
#' @param maxDist
#'   The maximum distance considered.
#' @param imageID The name of the colData column that stores in the image ID.
#' @param spatialCoords The columns that store the spatial coordinates.
#' @param cellType The name of the colData column that stores the cell types.
#' @param redDimName The name of the reduced dimension to store the distances in.
#' @param distFun What distance function to use. Can be min or abundance.
#' @param nCores Number of cores for parallel processing.
#' @examples
#' data("kerenSCE")
#'
#' kerenSCE <- getDistances(kerenSCE,
#'   maxDist = 200
#' )
#'
#' @export
#' @rdname getDistances
#' @importFrom dplyr
#'   bind_rows rename_with contains starts_with vars across
#' @importFrom spatstat.geom owin ppp closepairs
#' @importFrom BiocParallel bplapply MulticoreParam
#' @importFrom purrr reduce
#' @importFrom stringr str_replace
#' @importFrom SummarizedExperiment colData assayNames
#' @importFrom tibble column_to_rownames
#' @importFrom dplyr select contains mutate
#' @importFrom magrittr %>%
#' @importFrom S4Vectors metadata
#' @importFrom S4Vectors metadata<-
#' @importFrom SpatialExperiment spatialCoordsNames spatialCoords
getDistances <- function(cells,
                         maxDist = NULL,
                         imageID = "imageID",
                         spatialCoords = c("x", "y"),
                         cellType = "cellType",
                         redDimName = "distances",
                         distFun = "min",
                         nCores = 1) {
  x <- runif(1)
  BPPARAM <- .generateBPParam(cores = nCores)
  
  if (is.null(colnames(cells))) colnames(cells) <- seq_len(ncol(cells))

  if (!is(cells, "SingleCellExperiment")) stop("Currently this only accepts SpatialExperiment or SingleCellExperiment")


  
  if (is(cells, "SingleCellExperiment")) {
    cd <- cells |>
      SummarizedExperiment::colData() |>
      data.frame()
  }
  
  if (is(cells, "SpatialExperiment")) {
    cd <- cbind(colData(cells), SpatialExperiment::spatialCoords(cells)) |>
      data.frame()
    if(!all(spatialCoords%in%colnames(cd))) spatialCoords <- SpatialExperiment::spatialCoordsNames(cells)
  }
  
  
  if (!any(c(cellType, imageID, spatialCoords) %in% colnames(cd))) stop("Either imageID, cellType or spatialCoords is not in your colData")

  cd <- cd[, c(cellType, imageID, spatialCoords)]
  colnames(cd) <- c("cellType", "imageID", "x", "y")
  cd$cellID <- colnames(cells)


  cdFilt <- cd


  if (is.null(maxDist)) maxDist <- max(diff(range(cdFilt$x)), diff(range(cdFilt$y)))

  distances <- cdFilt |>
    split(~imageID) |>
    BiocParallel::bplapply(distanceCalculator,
      maxDist = maxDist,
      distFun = distFun,
      BPPARAM = BPPARAM
    )



  distances <- distances |>
    dplyr::bind_rows()

  SingleCellExperiment::reducedDim(cells, redDimName) <- distances[colnames(cells), ]

  cells
}






#' Wrapper to calculate imhomogenous K function between a cell and surrounding
#' types on each image
#'
#' Calculate the imhomogenous K function (a measure of cell type abundance) for
#' each cell to other cell types
#'
#' @param cells
#'   A dataframe with a cellType column as well as x and y spatial coordinates.
#'   The dataframe must contain a imageID column and cellID (unique cell
#'   identifier's) column as well
#' @param r
#'   Radius to include in that calculation of pairwise abundance (K-function)
#'   between cells (can be a numeric or vector of radii)
#' @param distFun What distance function to use.
#' @param redDimName Name of the reduced dimension to store in sce.
#' @param cellType The name of the column in colData that stores the cell types.
#' @param imageID The name of the column in colData that Stores the image ids.
#' @param spatialCoords The names of the columns in colData that store the spatial coordinates.
#' @param nCores Number of cores for parallel processing
#'
#'
#' @examples
#' library(dplyr)
#' data("kerenSCE")
#'
#' singleCellDataCounts <- getAbundances(kerenSCE,
#'   r = 200,
#' )
#'
#' @export
#' @rdname getAbundances
#' @importFrom dplyr bind_rows
#' @importFrom SingleCellExperiment reducedDim
#' @importFrom SummarizedExperiment colData assayNames
#' @importFrom BiocParallel bplapply
getAbundances <- function(cells,
                          r = 200,
                          distFun = "abundance",
                          redDimName = "abundances",
                          cellType = "cellType",
                          imageID = "imageID",
                          spatialCoords = c("x", "y"),
                          nCores = 1) {
  x <- runif(1)
  BPPARAM <- .generateBPParam(cores = nCores)

  if (!is(cells, "SingleCellExperiment")) stop("Currently this only accepts SpatialExperiment or SingleCellExperiment")

  if (is(cells, "SingleCellExperiment")) {
    cd <- cells |>
      SummarizedExperiment::colData() |>
      data.frame()
  }
  
  if (is(cells, "SpatialExperiment")) {
    cd <- cbind(colData(cells), SpatialExperiment::spatialCoords(cells)) |>
      data.frame()
    if(!all(spatialCoords%in%colnames(cd))) spatialCoords <- colnames(SpatialExperiment::spatialCoords(cells))
  }

  if (!any(c(cellType, imageID, spatialCoords) %in% colnames(cd))) stop("Either imageID, cellType or spatialCoords is not in your colData")

  cd <- cd[, c(cellType, imageID, spatialCoords)]
  colnames(cd) <- c("cellType", "imageID", "x", "y")
  if (is.null(colnames(cells))) colnames(cells) <- seq_len(ncol(cells))
  cd$cellID <- colnames(cells)

  # metadata_name <- paste0("Rs", Rs, "")

  cdFilt <- cd
  maxDist <- r

  if (is.null(maxDist)) maxDist <- max(diff(range(cdFilt$x)), diff(range(cdFilt$y)))

  distances <- cdFilt |>
    split(~imageID) |>
    BiocParallel::bplapply(distanceCalculator,
      maxDist = maxDist,
      distFun = distFun,
      BPPARAM = BPPARAM
    )



  distances <- distances |>
    dplyr::bind_rows()

  SingleCellExperiment::reducedDim(cells, redDimName) <- distances[colnames(cells), ]

  cells
}


#' Calculate the level of marker contamination of each cell
#'
#' Calculates contamination scores using a random forest classification
#'
#' @param cells
#'   A SingleCellExperiment or SpatialExperiment with a cellType column as well
#'   as marker intensity information corresponding to each cell.
#' @param markers
#'   A vector of markers that proxy a cell's state. If NULL, all markers
#'   will be used.
#' @param num.trees Number of trees to be used in the random forest classifier
#' @param verbose
#'   A logical indicating whether information about the final random forest
#'   model should be outputted.
#' @param missingReplacement
#'   A default value to replace missing marker intensities for classification.
#' @param assay
#'   The assay in the SingleCellExperiment object that contains the desired
#'   marker expressions.
#' @param cellType
#'   The name of the column in colData that stores the cell types.
#' @param redDimName The redDimName to store the output in the sce.
#'
#' @examples
#' data("kerenSCE")
#'
#' singleCellDataDistancesContam <- calcContamination(
#'   kerenSCE
#' )
#'
#' @export
#' @rdname calcContamination
#' @importFrom dplyr mutate select bind_rows across all_of
#' @importFrom SummarizedExperiment colData assayNames
#' @importFrom ranger ranger
#' @importFrom tibble column_to_rownames rownames_to_column
#' @importFrom stringr str_replace
#' @importFrom SingleCellExperiment reducedDim reducedDim<-
calcContamination <- function(cells,
                              markers = NULL,
                              num.trees = 100,
                              verbose = FALSE,
                              missingReplacement = 0,
                              assay = "intensities",
                              cellType = "cellType",
                              redDimName = "contaminations") {
  singleCellData <- cells
  if (is.null(markers)) {
    markers <- rownames(singleCellData)
  }


  if (!all(markers %in% colnames(colData(singleCellData)))) {
    singleCellDataClean <- singleCellData |>
      preProcessing(intensities = assay)
  } else {
    singleCellDataClean <- singleCellData
  }
  singleCellData <- as.data.frame(colData(singleCellDataClean))



  rfData <- singleCellData |>
    dplyr::select(cellType, all_of(markers)) |>
    dplyr::mutate(dplyr::across(any_of(markers), function(x) ifelse(is.nan(x) | is.na(x), 0, x)))

  rfData$cellType <- rfData[[cellType]]

  rfModel <- ranger::ranger(
    as.factor(cellType) ~ .,
    data = rfData,
    num.trees = num.trees,
    probability = TRUE
  )

  if (verbose == TRUE) {
    print(rfModel)
  }

  predictions <- predict(rfModel, rfData)$predictions

  maxn <- function(n) function(x) order(x, decreasing = TRUE)[!is.na(x)][n]

  rfData <- cbind(rfData, predictions)

  # check for duplicates - this can happen if your cells are annotated "CD4" for
  # "CD4 T cells" and you also have a gene marker "CD4"
  if (!is.empty(which(duplicated(names(rfData))))) {
    duplicate_columns <- which(duplicated(names(rfData)))
    print(paste("You have duplicate columns at columns ", paste0(duplicate_columns, collapse = ",")))
  }

  rfData <- rfData %>%
    dplyr::mutate(
      rfMaxCellProb = apply(
        .[colnames(predictions)],
        1,
        function(x) x[maxn(1)(x)]
      )
    )
  rfData <- rfData %>%
    dplyr::mutate(
      rfSecondLargestCellProb = apply(
        .[colnames(predictions)],
        1,
        function(x) x[maxn(2)(x)]
      )
    )
  rfData <- rfData %>%
    dplyr::mutate(
      rfMainCellProb = apply(
        .[c("cellType", colnames(predictions))],
        1,
        function(x) as.numeric(x[x["cellType"]])
      )
    ) %>%
    # dplyr::select(-colnames(predictions)) %>%
    tibble::rownames_to_column("cellID") %>%
    dplyr::mutate(cellID = stringr::str_replace(cellID, "cellID", "")) %>%
    select(-any_of(markers))

  # singleCellData2 <- singleCellData %>%
  #   dplyr::left_join(rfData, by = c("cellID"))

  redDim <- rfData
  rownames(redDim) <- colnames(cells)


  reducedDim(cells, redDimName) <- redDim

  return(cells)
}



#' First layer wrapper function to build linear models measuring state changes
#'
#' Builds linear models measuring marker based state changes in a cell type
#' based of the proximity or abundance of another cell type. The function
#' provides the option to build robust and mixed linear model variants
#'
#'
#'
#' @param cells
#'   A dataframe with a imageID, cellType, and marker intensity column along
#'   with covariates (e.g. distance or abundance of the nearest cell type) to
#'   model cell state changes
#' @param marker
#'  A vector of markers that proxy a cell's state. If NULL, all markers
#'  will be used.
#' @param from A vector of cell types to use as the primary cells. If NULL,
#'  all cell types  will be used.
#' @param to A vector of cell types to use as the interacting cells. If NULL,
#'  all cell types  will be used.
#' @param image A vector of images to filter to. If null all images will be used.
#' @param type What type of state change. This value should be in reduced dimensions.
#' @param assay
#'   The assay in the SingleCellExperiment object that contains the marker expressions.
#' @param cellType The column in colData that stores the cell types.
#' @param imageID The column in colData that stores the image ids.
#' @param contamination If TRUE, use the contamination scores that have previously
#'  been calculate. Otherwise a name of which reduced dimension contains the scores.
#' @param test  The type of test to perform. By default this will assume the data 
#' is Gaussian. A value of "nb" will use a negative binomial to model the expression.
#' @param minCells The minimum number of cells required to fit a model.
#' @param verbose A logical indicating if messages should be printed
#' @param timeout
#'   A maximum time allowed to build each model. Setting this may be important
#'   when building rlm mixed linear models
#' @param nCores Number of cores for parallel processing
#'
#' @examples
#' library(dplyr)
#' data("kerenSCE")
#'
#' kerenSCE <- kerenSCE[, kerenSCE$imageID %in% c(5, 6)]
#'
#' kerenSCE <- getDistances(kerenSCE,
#'   maxDist = 200,
#' )
#'
#' imageModels <- calcStateChanges(
#'   cells = kerenSCE,
#'   from = "Macrophages",
#'   to = "Tumour"
#' )
#'
#' @export
#' @rdname calcStateChanges
#' @importFrom BiocParallel bplapply MulticoreParam
#' @importFrom dplyr
#'   arrange group_by  summarise_at mutate bind_rows left_join filter left_join
#' @importFrom tidyr gather
#' @importFrom SummarizedExperiment colData assay
calcStateChanges <- function(cells,
                             marker = NULL,
                             from = NULL,
                             to = NULL,
                             image = NULL,
                             type = "distances",
                             assay = 1,
                             cellType = "cellType",
                             imageID = "imageID",
                             contamination = NULL,
                             test = "g",
                             minCells = 20,
                             verbose = FALSE,
                             timeout = 10,
                             nCores = 1) {
  if (is.null(marker)) {
    marker <- rownames(cells)
  }

  if (!is.null(image)) {
    cells <- cells[, colData(cells)[, imageID] %in% image]
  }

  if (is.null(to)) {
    to <- unique(colData(cells)[, cellType])
    if (class(to) == "factor") {
      to <- droplevels(to)
    }
  }

  if (is.null(from)) {
    from <- unique(colData(cells)[, cellType])
    if (class(from) == "factor") {
      from <- droplevels(from)
    }
  }

  if (!is.null(contamination)) {
    if (contamination == TRUE) contamination <- "contaminations"
  }

  
  minCellIdx <- cells |> 
    colData() |> 
    as.data.frame() |> 
    group_by(!!imageID, !!cellType) |> 
    mutate(indx = n() > minCells) |> 
    pull(indx)
  
  cells <- cells[, minCellIdx]
  
  cells <- cells[, colData(cells)[, cellType] %in% from]
  
  distances <- SingleCellExperiment::reducedDim(cells, type)
  distances <- distances[, to, drop = FALSE]
  intensities <- as.data.frame(t(as.matrix(SummarizedExperiment::assay(cells, assay))))
  intensities <- intensities[, marker, drop = FALSE]

  splitDist <- split(distances, ~ colData(cells)[, imageID] + colData(cells)[, cellType], sep = "51773")
  splitInt <- split(intensities, ~ colData(cells)[, imageID] + colData(cells)[, cellType], sep = "51773")
  if (is.null(contamination)) {
    contaminations <- data.frame(madeUp = rep(-99, ncol(cells)))
  } else {
    contaminations <- SingleCellExperiment::reducedDim(cells, contamination)
    contaminations <- dplyr::select(
      contaminations,
      -dplyr::any_of(c("cellID",
                       "cellType",
                       "rfMaxCellProb",
                       "rfSecondLargestCellProb",
                       "rfMainCellProb")))
    }

  splitCon <- split(contaminations, ~ colData(cells)[, imageID] + colData(cells)[, cellType], sep = "51773")

  use <- unlist(lapply(splitDist, nrow)) > minCells
  x <- runif(1)
  BPPARAM <- .generateBPParam(cores = nCores)


  allModels <- bpmapply(calculateChangesMarker,
    distances = splitDist[use],
    intensities = splitInt[use],
    contaminations = splitCon[use],
    MoreArgs = list(test = test),
    BPPARAM = BPPARAM,
    SIMPLIFY = FALSE
  )

  allModels <- dplyr::bind_rows(allModels, .id = "tmp")
  nam <- strsplit(allModels$tmp, 51773)
  allModels$primaryCellType <- unlist(lapply(nam, function(x) x[2]))
  allModels$imageID <- unlist(lapply(nam, function(x) x[1]))
  allModels$fdr <- stats::p.adjust(allModels$pval, "fdr")
  df <- dplyr::select(allModels, imageID, primaryCellType, otherCellType, marker, coef, tval, pval, fdr)
  df[order(df$pval), ]
}




#' @importFrom limma lmFit
#' @importFrom edgeR glmQLFTest glmQLFit DGEList topTags
calculateChangesMarker <- function (distances, intensities, contaminations, nCores, test) 
{
  testAll <- apply(distances, 2, function(x) {
    if (length(unique(x)) > 1) {
      if (contaminations[1, 1] == -99) {
        design <- data.frame(coef = 1, cellType = x)
      }
      else {
        contaminations <- contaminations[, !is.na(colSums(contaminations)), 
                                         drop = FALSE]
        # Decide whether to drop last column
        is_composition <- is.matrix(contaminations) &&
          all(abs(rowSums(contaminations) - 1) < 1e-8)
        
        if (is_composition) {
          contam_use <- contaminations[, -ncol(contaminations), drop = FALSE]
        } else {
          contam_use <- contaminations
        }
        
        # Build design
        design <- data.frame(coef = 1,
                             cellType = x,
                             contam_use)
        
        ## Drop zero-variance columns (constant 0/1 can also cause trouble)
        nzv <- vapply(as.data.frame(design), function(z) var(z) > 0, logical(1))
        nzv[1] <- TRUE
        design <- design[, nzv, drop = FALSE]
        
        ## Enforce full rank (drop aliased columns deterministically)
        qrX   <- qr(design)
        keep  <- qrX$pivot[seq_len(qrX$rank)]
        keep  <- c(intersect(1:2, keep), setdiff(keep, 1:2))
        design <- design[, keep, drop = FALSE]
      }
      if (any(is.na(design))) {
        design$cellType[is.na(design$cellType)] <- 0
      }
      exprs <- t(intensities)
      
      if(any(test != "nb")){
        
        fit <- Statial:::.quiet(limma::lmFit(exprs, design, trend = rep(1, 
                                                                        nrow(exprs)), verbose = FALSE))
        df <- data.frame(marker = colnames(intensities), 
                         coef = fit$coef[, "cellType"], tval = (fit$coef/fit$stdev.unscaled/fit$sigma)[, 
                                                                                                       "cellType"])
        df$pval <- 2 * pt(-abs(df$t), df = fit$df.residual)
        rownames(df) <- NULL
      }
      
      
      if(any(test == "nb")){
        y <- DGEList(counts=exprs)
        keep <- rowSums(y$counts) > 4
        y <- y[keep,, keep.lib.sizes=FALSE]
        keep <- colSums(y$counts) > 4
        y <- y[,keep, keep.lib.sizes=FALSE]
        design <- design[keep,]
        y$samples$lib.size     <- rep(1, ncol(y))
        
        df <- NULL
        if(sum(keep)>0){
          fit <- Statial:::.quiet(glmQLFit(y, as.matrix(design)))
          qlf <- glmQLFTest(fit, coef = 2)
          df1 <- topTags(qlf, n = Inf)
          df1 <- as.data.frame(df1)
          df <- data.frame(marker = rownames(df1), coef = df1$logFC, tval = abs(qnorm(df1$PValue, lower.tail = TRUE))*sign(df1$logFC), pval = df1$PValue)
        }
      }
      
      df
      
    }
  }, simplify = FALSE)
  testAll <- dplyr::bind_rows(testAll, .id = "otherCellType")
}

.quiet <- function(x, print_cat = TRUE, message = TRUE, warning = TRUE) {
  stopifnot(is.logical(print_cat) && length(print_cat) == 1)
  stopifnot(is.logical(message) && length(message) == 1)
  stopifnot(is.logical(warning) && length(warning) == 1)
  if (print_cat) {
    sink(tempfile(), type = "out")
  }
  on.exit(if (print_cat) sink())
  if (warning && message) {
    invisible(force(suppressMessages(suppressWarnings(x))))
  } else if (warning && !message) {
    invisible(suppressWarnings(force(x)))
  } else if (!warning && message) {
    invisible(suppressMessages(force(x)))
  } else {
    invisible(force(x))
  }
}






### Section 1: LM Calculations ##############################################################




### Section 4:Visualise Image ##############################################################

#' Visualise Cell-Cell Marker Relationships
#'
#' Helper functions to visualise OLS model fits for image based state models
#'
#' image,
#'
#' @param cells
#'   A SingleCellExperiment that has had distances already calculated.
#' @param type The name of the reduced dimension to use for the x-axis.
#' @param image An image to subset to.
#' @param imageID
#'   Identifier name of the image in the imageID column to be visualised
#' @param from
#'   A character indicating the name of the cell type (from the cellType column) whose cell state is being investigated in
#' @param to
#' A character indicating the name of the cell type (from the cellType column) who may be influencing the cell state of another cell type
#' @param marker
#'   The marker of interest.
#' @param assay Name of the assay that stores the marker expression.
#' @param cellType The name of the column in colData that stores the cell types.
#' @param imageID The name of the column in colData that stores the image ids.
#' @param spatialCoords The names of the columns in colData that store the spatial coordinates.
#' @param size
#'   Aesthetic numerical variable determining the size of the displayed cells
#' @param shape
#'   Aesthetic variable determining the shape grouping of the displayed cells
#' @param interactive
#'   Logical indicating if the output visualisation should be a interactive (plotly)
#' @param plotModelFit
#'   Logical indicating if fitted values should be plotted or actual intensities for marker specified. The default is to plot actual intensities
#' @param method
#'   The method to build the model with. Currently the only option is "lm". However, capabilities may be expanded in the future
#'
#' @examples
#' library(dplyr)
#' data("kerenSCE")
#'
#' kerenSCE <- getDistances(kerenSCE)
#'
#' p <- plotStateChanges(
#'   cells = kerenSCE,
#'   type = "distances",
#'   image = "6",
#'   from = "Keratin_Tumour",
#'   to = "Macrophages",
#'   marker = "p53",
#'   size = 1,
#'   shape = 19,
#'   interactive = FALSE,
#'   plotModelFit = FALSE,
#'   method = "lm"
#' )
#'
#' p
#'
#' @export
#' @rdname plotStateChanges
#' @importFrom dplyr filter left_join
#' @importFrom ggplot2
#'   ggplot scale_fill_distiller stat_density_2d geom_point theme_classic
#'   aes_string ggtitle facet_wrap aes xlab ylab ggtitle autoplot scale_colour_gradientn
#' @importFrom plotly ggplotly
#' @importFrom S4Vectors metadata
#' @importFrom S4Vectors metadata<-
#' @importFrom SingleCellExperiment reducedDimNames
plotStateChanges <- function(cells,
                             image,
                             from,
                             to,
                             marker,
                             type = "distances",
                             assay = 1,
                             cellType = "cellType",
                             imageID = "imageID",
                             spatialCoords = c("x", "y"),
                             size = 1,
                             shape = 19,
                             interactive = FALSE,
                             plotModelFit = FALSE,
                             method = "lm") {
  if (!imageID %in% colnames(colData(cells))) {
    stop("The argument imageID needs to exist in the data")
  }

  if (!image %in% colData(cells)[, imageID]) {
    stop("The image needs to exist in the data")
  }

  cells <- cells[, colData(cells)[, imageID] %in% image]

  if (!type %in% reducedDimNames(cells)) {
    stop("The reduced dimension needs to exist in the data")
  }

  if (is(cells, "SingleCellExperiment")) {
    cd <- cells |>
      SummarizedExperiment::colData() |>
      data.frame()
  }
  
  if (is(cells, "SpatialExperiment")) {
    cd <- cbind(colData(cells), SpatialExperiment::spatialCoords(cells)) |>
      data.frame()
    if(!all(spatialCoords%in%colnames(cd))) spatialCoords <- colnames(SpatialExperiment::spatialCoords(cells))
  }

  data <- data.frame(t(as.matrix(assay(cells, assay))), reducedDim(cells, type), cd)

  data$imageID <- data[, imageID]
  data$cellType <- data[, cellType]

  # data <- metadata(SCE)[[metadata_name]]

  if (!marker %in% colnames(data)) {
    stop("The marker needs to exist in the data")
  }

  if (!cellType %in% colnames(data)) {
    stop("cellType needs to exist in the colData")
  }

  if (!all(spatialCoords %in% colnames(data))) {
    stop("spatialCoords needs to exist in the colData")
  }

  data$x <- data[, spatialCoords[1]]
  data$y <- data[, spatialCoords[2]]

  data$marker <- data[, marker, drop = TRUE]
  data$fittedValues <- NA

  relationshipFormula <- paste0(
    marker, "~", to
  )

  modelData <- data[data[, cellType] == from, ]
  model <- lm(formula(relationshipFormula), modelData)

  # print(summary(model))

  data[data[, cellType] == from, "fittedValues"] <- predict(
    model,
    data[data$cellType == from, ]
  )


  if (plotModelFit == TRUE) {
    data[data$cellType == from, marker] <- predict(
      model, data[data$cellType == from, ]
    )
  }



  g1 <- ggplot2::ggplot() +
    ggplot2::stat_density_2d(
      data = data[data$cellType == to, ],
      ggplot2::aes(
        x = x, y = y, fill = ggplot2::after_stat(density)
      ),
      geom = "raster",
      contour = FALSE
    ) +
    ggplot2::geom_point(data = data[data$cellType == to, ], aes(x, y), size = size / 2, colour = "darkblue") +
    ggplot2::scale_fill_distiller(palette = "Blues", direction = 1) +
    ggplot2::geom_point(
      data = data[data$cellType == from, ],
      ggplot2::aes_string(
        x = "x", y = "y",
        colour = marker
      ),
      size = size,
      shape = shape
    ) +
    ggplot2::theme_classic() +
    ggplot2::ggtitle(paste(
      "Cell Points:", from, ",",
      "Cell Density:", to, ",",
      "Model Fit:", plotModelFit
    )) +
    ggplot2::facet_wrap(~imageID, scales = "free") +
    scale_colour_gradientn(colours = rep(c("black", "darkred", "red", "orange", "yellow"), c(1, 3, 3, 3, 3)))


  g2 <- data %>%
    dplyr::filter(cellType == from) |>
    ggplot2::ggplot(
      ggplot2::aes_string(
        x = to,
        y = marker
      )
    ) +
    ggplot2::geom_point() +
    ggplot2::geom_smooth(method = lm) +
    ggplot2::theme_classic() +
    ggplot2::ggtitle("State Change Scatter Plot") +
    ggplot2::ylab(paste(marker, "expression in ", from)) +
    ggplot2::xlab(paste(from, "cell distance from ", to)) +
    ggplot2::ylim(-1, NA)



  g3 <- data |>
    dplyr::filter(cellType == from) |>
    ggplot2::ggplot(ggplot2::aes_string(
      x = marker,
      y = "fittedValues"
    )) +
    ggplot2::geom_point() +
    ggplot2::geom_smooth(method = lm) +
    ggplot2::theme_classic() +
    ggplot2::xlab("True Values") +
    ggplot2::ylab("Fitted Values") +
    ggplot2::ggtitle("Predicted vs Real Values")

  # g4 <- ggplot2::autoplot(model) + ggplot2::theme_classic()

  if (interactive == TRUE) {
    g1 <- plotly::ggplotly(g1)
    g2 <- plotly::ggplotly(g2)
    g3 <- plotly::ggplotly(g3)
  }
  list(image = g1, scatter = g2)
  # list(g1, g2, g3, g4)
}




#' Extract the average expression for all markers for each cell type in each
#' region defined by lisaClust
#'
#' Takes a SingleCellExperiment and outputs a dataframe in a convenient format for
#' cross validation
#'
#'
#' @param data
#'   A SingleCellExperiment object with intensities data in the assays slot and
#'   regions information in colData generated by lisaClust.
#' @param imageID
#'   The colData column that stores the image IDs.
#' @param cellType
#'   The colData column that store the cell types.
#' @param region
#'   The colData column that stores the regions.
#' @param markers
#'   A string vector of markers that proxy a cell's state. If NULL, all markers
#'   will be used.
#' @param assay Which assay do you want to use for the expression data.
#' @param replaceVal A value to replace missing values with.
#'
#' @examples
#' data(kerenSCE)
#'
#' kerenSCE <- kerenSCE[, kerenSCE$imageID %in% c("5", "6")]
#'
#' regionSCE <- lisaClust::lisaClust(kerenSCE, k = 5)
#'
#' lisaClustOutput <- getMarkerMeans(regionSCE)
#' @export
#' @rdname getMarkerMeans
#' @importFrom dplyr left_join group_by
#' @importFrom tidyr pivot_wider pivot_longer
#' @importFrom SummarizedExperiment colData assay
#' @importFrom tibble column_to_rownames
getMarkerMeans <- function(data,
                           imageID = NULL,
                           cellType = NULL,
                           region = NULL,
                           markers = NULL,
                           assay = 1,
                           replaceVal = 0) {
  if (is.null(markers)) {
    markers <- rownames(data)
  }

  if (!is.null(imageID)) {
    if (!imageID %in% colnames(colData(data))) stop("Your imageID is not in colData")
    data$imageID <- colData(data)[, imageID]
    imageID <- "imageID"
  }

  if (!is.null(cellType)) {
    if (!cellType %in% colnames(colData(data))) stop("Your cellType is not in colData")
    data$cellType <- colData(data)[, cellType]
    cellType <- "cellType"
  }

  if (!is.null(region)) {
    if (!region %in% colnames(colData(data))) stop("Your region is not in colData")
    data$region <- colData(data)[, region]
    region <- "region"
  }



  use <- c(imageID, cellType, region)

  df <- data.frame(colData(data)[, use, drop = FALSE], t(assay(data, assay)))
  df <- tidyr::pivot_longer(df, -any_of(use), names_to = "markers")
  df <- dplyr::group_by(df, across(-value))

  if (!is.null(imageID)) {
    m <- tidyr::pivot_wider(df, names_from = c(markers, cellType, region), values_from = value, values_fn = mean, names_sep = "__", values_fill = replaceVal)
    m <- column_to_rownames(m, "imageID")
    m <- as.data.frame(m)
  }

  if (is.null(imageID)) {
    m <- tidyr::pivot_wider(df, names_from = markers, values_from = value, values_fn = mean, names_sep = "__", values_fill = replaceVal)
    m <- as.data.frame(m)
  }
  m
}

#' Calculates L function
#' @noRd
.Lfunction <- function(closePairs, counts, child1, child2, r, area) {
  
  # Define counts
  nChild1 <- sum(counts$cellTypeI == child1)
  nChild2 <- sum(counts$cellTypeI == child2)

  # Adds up number of cell type 2 within cell type
  numerator <- sum(counts[cellTypeI == child1, get(child2)])

  lambda2 <- nChild2 / area
  kvalue <- (numerator / lambda2) * (1 / nChild1)

  centeredL = sqrt(kvalue / pi) - r
  
  return(centeredL)
}


#' Calculates inhomogenous L function
#' @noRd
#' @importFrom data.table ":="
.Linhomfunction <- function(closePairs, counts, child1, child2, r, area) {
  
  # Defining counts 
  nChild1 <- sum(counts$cellTypeI == child1)
  nChild2 <- sum(counts$cellTypeI == child2)

  # Local intensity of each point
  weightChild1 <- counts[[child1]] / (pi * r^2)
  weightChild2 <- counts[[child2]] / (pi * r^2)
  
  # Define closePairs
  closePairsShort <- closePairs[cellTypeI == child1 & cellTypeJ == child2, ]
  closePairsShort[, weightParent := edge / (weightChild1[i] * weightChild2[j])]
  
  # Calculate numerator
  numerator <- sum(closePairsShort$weightParent)
  denominator <- sum(1 / weightChild1[counts$cellTypeI == child1])
  
  centeredL = sqrt((numerator / denominator) / pi) - r
  
  return(centeredL)
}




#' Calculates kontextual value
#' @noRd
#' @importFrom data.table ":=" .SD
.Kontext <- function(closePairs, counts, child1, child2, parent, r, returnWeight) {
  # child1 is root cell
  # child2 is child cell
  
  # Defining counts
  nParent <- sum(counts$cellTypeI %in% parent)
  nChild1 <- sum(counts$cellTypeI == child1)
  nChild2 <- sum(counts$cellTypeI == child2)

  # Defining lambdas
  parentNames = parent[parent %in% names(counts)]
  countsParent <- counts[, rowSums(.SD), .SDcols = parentNames]
  lambdaParent <- countsParent / (pi * r^2)
  lambdaChild1 <- lambdaParent
  lambdaChild2 <- (lambdaParent / nParent) * nChild2
  
  closePairsShort <- closePairs[cellTypeI == child1 & cellTypeJ == child2, ]
  closePairsShort[, weightParent := (edge * lambdaChild1[i]) / (lambdaChild2[j])]
  
  
  numerator <- sum(closePairsShort$weightParent)
  # weighted mean over child 2
  denominator <- sum(lambdaChild1[counts$cellTypeI == child1])
  
  # Turn this into centered L
  centeredL = sqrt(numerator / denominator / pi) - r
  
  return(centeredL)

}





#' Converts a PPP object to a data frame for inhomLParent function
#' @noRd
#' @import spatstat.geom
#'
PPPdf <- function(ppp) {
  x <- as.data.frame(ppp)
  x$cellType <- factor(x$marks)
  x$x <- x$x
  x$y <- x$y
  x$cellID <- factor(seq_len(nrow(x)))
  return(x)
}


#' Edge correction for L values
#' @noRd
#' @import spatstat.geom

.borderEdge <- function(X, maxD) {
  W <- X$window
  bW <- spatstat.geom::union.owin(
    spatstat.geom::border(W, maxD, outside = FALSE),
    spatstat.geom::border(W, 2, outside = TRUE)
  )
  inB <- spatstat.geom::inside.owin(X$x, X$y, bW)
  e <- rep(1, X$n)
  if (any(inB)) {
    circs <- spatstat.geom::discs(X[inB], maxD, separate = TRUE)
    circs <-
      spatstat.geom::solapply(circs, spatstat.geom::intersect.owin, X$window)
    areas <- unlist(lapply(circs, spatstat.geom::area)) / (pi * maxD^2)
    e[inB] <- areas
  }

  e
}

---
title: "Statial"
date: "`r BiocStyle::doc_date()`"
params:
  test: FALSE
author:
- name: Farhan Ameen
  affiliation:  
  - School of Mathematics and Statistics, University of Sydney, Australia
- name: Sourish Iyengar
  affiliation:
  - School of Mathematics and Statistics, University of Sydney, Australia
- name: Alex Qin
  affiliation:  
  - School of Mathematics and Statistics, University of Sydney, Australia
- name: Ellis Patrick
  affiliation:
  - &WIMR Westmead Institute for Medical Research, University of Sydney, Australia
  - School of Mathematics and Statistics, University of Sydney, Australia
vignette: >
  %\VignetteIndexEntry{"Introduction to Statial"}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output: 
  BiocStyle::html_document
---

```{r chunk setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  cache = TRUE,
  message = FALSE,
  warning = FALSE
)
library(BiocStyle)
```

# Installation

```{r installation, eval = FALSE}
# Install the package from Bioconductor
if (!requireNamespace("BiocManager", quietly = TRUE)) {
  install.packages("BiocManager")
}
BiocManager::install("Statial")
```

# Load packages

```{r libraries and setup, warning = FALSE, message = FALSE, eval = TRUE}
# Loading required packages
library(Statial)
library(spicyR)
library(ClassifyR)
library(lisaClust)
library(dplyr)
library(SingleCellExperiment)
library(ggplot2)
library(ggsurvfit)
library(survival)
library(tibble)
library(treekoR)

theme_set(theme_classic())
nCores <- 1
```

# Overview

There are over 37 trillion cells in the human body, each taking up different forms and functions. The behaviour of these cells can be described by canonical characteristics, but their functions can also dynamically change based on their environmental context. Understanding the interplay between cells is key to understanding their mechanisms of action and how they contribute to human disease. `Statial` is a suite of functions to quantify the spatial relationships between cell types. This guide will provide a step-by-step overview of some key functions within `Statial`.

# Loading example data

To illustrate the functionality of Statial we will use a multiplexed ion beam imaging by time-of-flight (MIBI-TOF) dataset profiling tissue from triple-negative breast cancer patients$^1$ by Keren et al., 2018. This dataset simultaneously quantifies *in situ* expression of 36 proteins in 34 immune rich patients. *Note:* The data contains some "uninformative" probes and the original cohort included 41 patients.

These images are stored in a `SingleCellExperiment` object called `kerenSCE`. This object contains 57811 cells across 10 images and includes information on cell type and patient survival.

*Note:* The original dataset was reduced down from 41 images to 10 images for the purposes of this vignette, due to size restrictions.

```{r load dataset}
# Load breast cancer
data("kerenSCE")

kerenSCE
```

# Kontextual: Context aware spatial relationships

`Kontextual` is a method for performing inference on cell localisation which explicitly defines the contexts in which spatial relationships between cells can be identified and interpreted. These contexts may represent landmarks, spatial domains, or groups of functionally similar cells which are consistent across regions. By modelling spatial relationships between cells relative to these contexts, `Kontextual` produces robust spatial quantifications that are not confounded by biases such as the choice of region to image and the tissue structure present in the images.

In this example we demonstrate how cell type hierarchies can be used as a means to derive appropriate "contexts" for the evaluation of cell localisation. We then demonstrate the types of conclusions which `Kontextual` enables.

## Using cell type hierarchies to define a "context"

A cell type hierarchy may be used to define the "context" in which cell type relationships are evaluated within. A cell type hierarchy defines how cell types are functionally related to one another. The bottom of the hierarchy represents homogeneous populations of a cell type (child), the cell populations at the nodes of the hierarchy represent broader parent populations with shared generalised function. For example CD4 T cells may be considered a child population to the Immune parent population.

There are two ways to define the cell type hierarchy. First, they can be defined based on our biological understanding of the cell types. We can represent this by creating a named list containing the names of each parent and the associated vector of child cell types.

*Note:* The `all` vector must be created to include cell types which do not have a parent e.g. the *undefined* cell type in this data set.

```{r biological hierarchy}
# Examine all cell types in image
unique(kerenSCE$cellType)

# Named list of parents and their child cell types
biologicalHierarchy = list(
  "tumour" = c("Keratin_Tumour", "Tumour"),
  "tcells" = c("CD3_Cell", "CD4_Cell", "CD8_Cell", "Tregs"),
  "myeloid" = c("Dc/Mono", "DC", "Mono/Neu", "Macrophages", "Neutrophils"),
  "tissue" = c("Endothelial", "Mesenchymal")
)

# Adding more broader immune parent populationse
biologicalHierarchy$immune = c(biologicalHierarchy$bcells,
                               biologicalHierarchy$tcells,
                               biologicalHierarchy$myeloid,
                               "NK", "other immune", "B")


# Creating a vector for all cellTypes
all <- unique(kerenSCE$cellType)
```

Alternatively, you can use the `treeKor` bioconductor package [treekoR](http://www.bioconductor.org/packages/release/bioc/html/treekoR.html) to define these hierarchies in a data driven way.

*Note:* These parent populations may not be accurate as we are using a small subset of the data. 

```{r clustering hierarchy, warning = FALSE}
# Calculate hierarchy using treekoR
kerenTree <- treekoR::getClusterTree(t(assay(kerenSCE, "intensities")),
                            kerenSCE$cellType,
                            hierarchy_method="hopach",
                            hopach_K = 1)

# Convert treekoR result to a name list of parents and children.
treekorParents = getParentPhylo(kerenTree)

treekorParents
```

## Application on triple negative breast cancer image.

Here we examine an image highlighted in the Keren et al. 2018 manuscript where accounting for context information enables new conclusions. In image 6 of the Keren et al. dataset, we can see that *p53+ tumour cells* and *immune cells* are dispersed i.e. these two cell types are not mixing. However we can also see that *p53+ tumour cells* appear much more localised to *immune cells* relative to the tumour context (*tumour cells* and *p53+ tumour cells*).

```{r image6}
# Lets define a new cell type vector
kerenSCE$cellTypeNew <- kerenSCE$cellType

# Select for all cells that express higher than baseline level of p53
p53Pos <- assay(kerenSCE)["p53", ] > -0.300460

# Find p53+ tumour cells
kerenSCE$cellTypeNew[kerenSCE$cellType %in% biologicalHierarchy$tumour] <- "Tumour"
kerenSCE$cellTypeNew[p53Pos & kerenSCE$cellType %in% biologicalHierarchy$tumour] <- "p53_Tumour"

# Group all immune cells under the name "Immune"
kerenSCE$cellTypeNew[kerenSCE$cellType %in% biologicalHierarchy$immune] <- "Immune"

# Plot image 6
kerenSCE |>
  colData() |>
  as.data.frame() |>
  filter(imageID == "6") |>
  filter(cellTypeNew %in% c("Immune", "Tumour", "p53_Tumour")) |>
  arrange(cellTypeNew) |>
  ggplot(aes(x = x, y = y, color = cellTypeNew)) +
  geom_point(size = 1) +
  scale_colour_manual(values = c("Immune" = "#505050", "p53_Tumour" = "#64BC46", "Tumour" = "#D6D6D6")) +
  guides(colour = guide_legend(title = "Cell types", override.aes = list(size = 3)))
```

`Kontextual` accepts a `SingleCellExperiment` object, a single image, or list of images from a `SingleCellExperiment` object, which gets passed into the `cells` argument. The two cell types which will be evaluated are specified in the `to` and `from` arguments. A parent population must also be specified in the `parent` argument, note the parent cell population must include the `to` cell type. The argument `r` will specify the radius which the cell relationship will be evaluated on. `Kontextual` supports parallel processing, the number of cores can be specified using the `cores` argument. `Kontextual` can take a single value or multiple values for each argument and will test all combinations of the arguments specified.

We can calculate these relationships across all images for a single radius (r = 100). Small radii will examine local spatial relationships, whereas larger radii will examine global spatial relationships.

```{r p53 relationship}
p53_Kontextual <- Kontextual(
  cells = kerenSCE,
  r = 100,
  from = "Immune",
  to = "p53_Tumour",
  parent = c("p53_Tumour", "Tumour"),
  cellType = "cellTypeNew"
)

p53_Kontextual
```

The `kontextCurve` function plots the L-function value and Kontextual values over a range of radii. If the points lie above the red line (expected pattern) then localisation is indicated for that radius, if the points lie below the red line then dispersion is indicated. 

As seen in the following plot the L-function produces negative values over a range of radii, indicating that *p53+ tumour cells* and *immune cells* are dispersed from one another. However by taking into account the tumour context, `Kontextual` shows positive values over some radii, indicating localisation between *p53+ tumour cells* and *immune cells*.

```{r kontextCurve}
curves <- kontextCurve(
  cells = kerenSCE,
  from = "Immune",
  to = "p53_Tumour",
  parent = c("p53_Tumour", "Tumour"),
  rs = seq(50, 510, 50),
  image = "6",
  cellType = "cellTypeNew",
  cores = nCores
)

kontextPlot(curves)
```

Alternatively all pairwise cell relationships and their corresponding parent in the dataset can be tested. A data frame with all pairwise combinations can be creating using the `parentCombinations` function. This function takes in a vector of all the cells, as well as the named list of parents and children created earlier in the `parentList` argument. As shown below the output is a data frame specifying the `to`, `from`, and `parent` arguments for `Kontextual`.

*Note:* the output of `getPhyloParent` may also be using the in the `parentList` argument, for example if you wanted to use the treekoR defined hierarchy instead.

```{r parentDf}
# Get all relationships between cell types and their parents
parentDf <- parentCombinations(
  all = all,
  parentList = biologicalHierarchy
)
```


## Calculating all pairwise relationships

Rather than specifying `to`, `from`, and `parent` in `Kontextual`, the output from `parentCombinations` can be inputed into `Kontextual` using the `parentDf` argument, to examine all pairwise relationships in the dataset. This chunk will take a significant amount of time to run, for demonstration the results have been saved and are loaded in.

```{r kontextual}
# Running Kontextual on all relationships across all images.
kerenKontextual <- Kontextual(
  cells = kerenSCE,
  parentDf = parentDf,
  r = 100,
  cores = nCores
)
```

For every pairwise relationship (named accordingly: `from__to__parent`) `Kontextual` outputs the L-function values (original) and the Kontextual value. The relationships where the L-function and Kontextual disagree (e.g. one metric is positive and the other is negative) represent relationships where adding context information results in different conclusions on the spatial relationship between the two cell types.

```{r head kontextual results}
head(kerenKontextual, 10)
```

## Associating the relationships with survival outcomes.

To examine whether the features obtained from `Statial` are associated with patient outcomes or groupings, we can use the `spicy` function from the `spicyR` package. `spicy` requires the `SingleCellExperiment` object being used to contain a column called `survival`.

```{r survival data}
# add survival column to kerenSCE
kerenSCE$event = 1 - kerenSCE$Censored
kerenSCE$survival = Surv(kerenSCE$Survival_days_capped, kerenSCE$event)
```


In addition to this, the Kontextual results must be converted from a `data.frame` to a wide `matrix`, this can be done using `prepMatrix`. Note, to extract the original L-function values, specify `column = "original"` in `prepMatrix`.

```{r prep kontextual matrix}
# Converting Kontextual result into data matrix
kontextMat <- prepMatrix(kerenKontextual)

# Ensuring rownames of kontextMat match up with the image IDs of the SCE object
kontextMat <- kontextMat[kerenSCE$imageID |> unique(), ]

# Replace NAs with 0
kontextMat[is.na(kontextMat)] <- 0
```

Finally, both the `SingleCellExperiment` object and the Kontextual matrix are passed into the `spicy` function, with `condition = "survival"`. The resulting coefficients and p values can be obtained by accessing the `survivalResults` name.


*Note:* You can specify additional covariates and include a subject id for mixed effects survival modelling, see \code{\link[spicyR]{spicy} for more information.

```{r survival anaylysis}
# Running survival analysis
survivalResults = spicy(cells = kerenSCE,
                        alternateResult = kontextMat,
                        condition = "survival",
                        weights = TRUE)

head(survivalResults$survivalResults, 10)

```

The survival results can also be visualised using the `signifPlot` function.

```{r survival signifPlot}
signifPlot(survivalResults)
```


As we can see from the results `DC__NK__immune` is the one of the most significant pairwise relationship which contributes to patient survival. That is the relationship between dendritic cells and natural killer cells, relative to the parent population of immune cells. We can see that there is a negative coefficient associated with this relationship, which tells us an increase in localisation of these cell types relative to immune cells leads to better survival outcomes for patients.

The association between `DC__NK__immune` and survival can also be visualised on a Kaplan-Meier curve. First, we extract survival data from the `SingleCellExperiment` object and create a survival vector. 


```{r extract data KM curve}
# Extracting survival data
survData <- kerenSCE |>
  colData() |>
  data.frame() |>
  select(imageID, survival) |>
  unique()

# Creating survival vector
kerenSurv <- survData$survival
names(kerenSurv) <- survData$imageID

kerenSurv
```

Next, we extract the Kontextual values of this relationship across all images. We then determine if dendritic and natural killer cells are relatively attracted or avoiding in each image by comparing the Kontextual value in each image to the median Kontextual value.

Finally, we plot a Kaplan-Meier curve using the `ggsurvfit` package. As shown below, when dendritic and natural killer cells are more localised to one another relative to the immune cell population, patients tend to have better survival outcomes.

```{r KM curve, fig.width=5, fig.height=4}
# Selecting most significant relationship
survRelationship <- kontextMat[["DC__NK__immune"]]
survRelationship <- ifelse(survRelationship > median(survRelationship), "Localised", "Dispersed")

# Plotting Kaplan-Meier curve
survfit2(kerenSurv ~ survRelationship) |>
  ggsurvfit() +
  ggtitle("DC__NK__immune")
```

# SpatioMark: Identifying continuous changes in cell state

Changes in cell states can be analytically framed as the change in abundance of a gene or protein within a particular cell type. We can use marker expression to identify and quantify evidence of cell interactions that catalyse cell state changes. This approach measures how protein markers in a cell change with spatial proximity and abundance to other cell types. The methods utilised here will thereby provide a framework to explore how the dynamic behaviour of cells are altered by the agents they are surrounded by.

## Continuous cell state changes within a single image

The first step in analysing these changes is to calculate the spatial proximity (`getDistances`) and abundance (`getAbundances`) of each cell to every cell type. These values will then be stored in the `reducedDims` slot of the `SingleCellExperiment` object under the names `distances` and `abundances` respectively.

```{r calc metrics}
kerenSCE <- getDistances(kerenSCE,
  maxDist = 200,
  nCores = 1
)

kerenSCE <- getAbundances(kerenSCE,
  r = 200,
  nCores = 1
)
```

First, let's examine the same effect observed earlier with Kontextual - the localisation between p53-positive keratin/tumour cells and macrophages in the context of total keratin/tumour cells for image 6 of the Keren et al. dataset.

Statial provides two main functions to assess this relationship - `calcStateChanges` and `plotStateChanges`. We can use `calcStateChanges` to examine the relationship between 2 cell types for 1 marker in a specific image. In this case, we're examining the relationship between keratin/tumour cells (`from = Keratin_Tumour`) and macrophages (`to = "Macrophages"`) for the marker p53 (`marker = "p53"`) in `image = "6"`. We can appreciate that the `fdr` statistic for this relationship is significant, with a negative tvalue, indicating that the expression of p53 in keratin/tumour cells decreases as distance from macrophages increases.

```{r calcStateChanges}
stateChanges <- calcStateChanges(
  cells = kerenSCE,
  type = "distances",
  image = "6",
  from = "Keratin_Tumour",
  to = "Macrophages",
  marker = "p53",
  nCores = 1
)

stateChanges
```

Statial also provides a convenient function for visualising this interaction - `plotStateChanges`. Here, again we can specify `image = 6` and our main cell types of interest, keratin/tumour cells and macrophages, and our marker p53, in the same format as `calcStateChanges`.

Through this analysis, we can observe that keratin/tumour cells closer to a group of macrophages tend to have higher expression of p53, as observed in the first graph. This relationship is quantified with the second graph, showing an overall decrease of p53 expression in keratin/tumour cells as distance to macrophages increase.

These results allow us to essentially arrive at the same result as Kontextual, which calculated a localisation between p53+ keratin/tumour cells and macrophages in the wider context of keratin/tumour cells.

```{r plotStateChanges p53}
p <- plotStateChanges(
  cells = kerenSCE,
  type = "distances",
  image = "6",
  from = "Keratin_Tumour",
  to = "Macrophages",
  marker = "p53",
  size = 1,
  shape = 19,
  interactive = FALSE,
  plotModelFit = FALSE,
  method = "lm"
)

p
```

## Continuous cell state changes across all images

Beyond looking at single cell-to-cell interactions for a single image, we can also look at all interactions across all images. The `calcStateChanges` function provided by Statial can be expanded for this exact purpose - by not specifying cell types, a marker, or an image, `calcStateChanges` will examine the most significant correlations between distance and marker expression across the entire dataset. Here, we've filtered out the most significant interactions to only include those found within image 6 of the Keren et al. dataset.

```{r calcStateChanges all}
stateChanges <- calcStateChanges(
  cells = kerenSCE,
  type = "distances",
  nCores = 1,
  minCells = 100
)

stateChanges |>
  filter(imageID == 6) |>
  head(n = 10)
```

In image 6, the majority of the top 10 most significant interactions occur between keratin/tumour cells and an immune population, and many of these interactions appear to involve the HLA class I ligand.

We can examine some of these interactions further with the `plotStateChanges` function. Taking a closer examination of the relationship between macrophages and keratin/tumour HLA class I expression, the plot below shows us a clear visual correlation - as macrophage density increases, keratin/tumour cells increase their expression HLA class I.

Biologically, HLA Class I is a ligand which exists on all nucleated cells, tasked with presenting internal cell antigens for recognition by the immune system, marking aberrant cells for destruction by either CD8+ T cells or NK cells.

```{r plotStateChanges HLA}
p <- plotStateChanges(
  cells = kerenSCE,
  type = "distances",
  image = "6",
  from = "Keratin_Tumour",
  to = "Macrophages",
  marker = "HLA_Class_1",
  size = 1,
  shape = 19,
  interactive = FALSE,
  plotModelFit = FALSE,
  method = "lm"
)

p
```

Next, let's take a look at the top 10 most significant results across all images.

```{r statChanges}
stateChanges |> head(n = 10)
```

Immediately, we can appreciate that a couple of these interactions are not biologically plausible. One of the most significant interactions occurs between B cells and CD4 T cells in image 35, where CD4 T cells are found to increase in CD20 expression when in close proximity to B cells. Biologically, CD20 is a highly specific ligand for B cells, and under healthy circumstances are usually not expressed in T cells.

Could this potentially be an artefact of `calcStateChanges`? We can examine the image through the `plotStateChanges` function, where we indeed observe a strong increase in CD20 expression in T cells nearby B cell populations.

```{r plotStateChanges CD20}
p <- plotStateChanges(
  cells = kerenSCE,
  type = "distances",
  image = "35",
  from = "CD4_Cell",
  to = "B",
  marker = "CD20",
  size = 1,
  shape = 19,
  interactive = FALSE,
  plotModelFit = FALSE,
  method = "lm"
)

p
```

So why are T cells expressing CD20? This brings us to a key problem of cell segmentation - contamination.

## Contamination (Lateral marker spill over)

Contamination, or lateral marker spill over is an issue that results in a cell’s marker expressions being wrongly attributed to another adjacent cell. This issue arises from incorrect segmentation where components of one cell are wrongly determined as belonging to another cell. Alternatively, this issue can arise when antibodies used to tag and measure marker expressions don't latch on properly to a cell of interest, thereby resulting in residual markers being wrongly assigned as belonging to a cell near the intended target cell. It is important that we either correct or account for this incorrect attribution of markers in our modelling process. This is critical in understanding whether significant cell-cell interactions detected are an artefact of technical measurement errors driven by spill over or are real biological changes that represent a shift in a cell’s state.

To circumvent this problem, Statial provides a function that predicts the probability that a cell is any particular cell type - `calcContamination`. `calcContamination` returns a dataframe of probabilities demarcating the chance of a cell being any particular cell type. This dataframe is stored under `contaminations` in the `reducedDim` slot of the `SingleCellExperiment` object. It also provides the `rfMainCellProb` column, which provides the probability that a cell is indeed the cell type it has been designated. E.g. For a cell designated as CD8, rfMainCellProb could give a 80% chance that the cell is indeed CD8, due to contamination.

We can then introduce these probabilities as covariates into our linear model by setting `contamination = TRUE` as a parameter in our `calcStateChanges` function. However, this is not a perfect solution for the issue of contamination. As we can see, despite factoring in contamination into our linear model, the correlation between B cell density and CD20 expression in CD4 T cells remains one of the most significant interactions in our model.

```{r calcContamination}
kerenSCE <- calcContamination(kerenSCE)

stateChangesCorrected <- calcStateChanges(
  cells = kerenSCE,
  type = "distances",
  nCores = 1,
  minCells = 100,
  contamination = TRUE
)

stateChangesCorrected |> head(n = 20)
```

However, this does not mean factoring in contamination into our linear model was ineffective.

Whilst our correction attempts do not rectify every relationship which arises due to contamination, we show that a significant portion of these relationships are rectified. We can show this by plotting a ROC curve of true positives against false positives. In general, cell type specific markers such as CD4, CD8, and CD20 should not change in cells they are not specific to. Therefore, relationships detected to be significant involving these cell type markers are likely false positives and will be treated as such for the purposes of evaluation. Meanwhile, cell state markers are predominantly likely to be true positives.

Plotting the relationship between false positives and true positives, we'd expect the contamination correction to be greatest in the relationships with the top 100 lowest p values, where we indeed see more true positives than false positives with contamination correction.

```{r corrected}
cellTypeMarkers <- c("CD3", "CD4", "CD8", "CD56", "CD11c", "CD68", "CD45", "CD20")

values <- c("blue", "red")
names(values) <- c("None", "Corrected")

df <- rbind(
  data.frame(TP = cumsum(stateChanges$marker %in% cellTypeMarkers), FP = cumsum(!stateChanges$marker %in% cellTypeMarkers), type = "None"),
  data.frame(TP = cumsum(stateChangesCorrected$marker %in% cellTypeMarkers), FP = cumsum(!stateChangesCorrected$marker %in% cellTypeMarkers), type = "Corrected")
)

ggplot(df, aes(x = TP, y = FP, colour = type)) +
  geom_line() +
  labs(y = "Cell state marker", x = "Cell type marker") +
  scale_colour_manual(values = values)
```

Here, we zoom in on the ROC curve where the top 100 lowest p values occur, where we indeed see more true positives than false positives with contamination correction.

```{r corrected zoom}
ggplot(df, aes(x = TP, y = FP, colour = type)) +
  geom_line() +
  xlim(0, 100) +
  ylim(0, 1000) +
  labs(y = "Cell state marker", x = "Cell type marker") +
  scale_colour_manual(values = values)
```

## Associate continuous state changes with survival outcomes

Similiar to `Kontextual`, we can run a similar survival analysis using our state changes results. Here, `prepMatrix` extracts the coefficients, or the `coef` column of `stateChanges` by default. To use the t values instead, specify `column = "tval"` in the `prepMatrix` function.

```{r survival state changes}
# Preparing features for Statial
stateMat <- prepMatrix(stateChanges)

# Ensuring rownames of stateMat match up with rownames of the survival vector
stateMat <- stateMat[names(kerenSurv), ]

# Remove some very small values
stateMat <- stateMat[, colMeans(abs(stateMat) > 0.0001) > .8]

survivalResults <- colTest(stateMat, kerenSurv, type = "survival")

head(survivalResults)
```

For our state changes results, `Keratin_Tumour__CD4_Cell__Keratin6` is the most significant pairwise relationship which contributes to patient survival. That is, the relationship between HLA class I expression in keratin/tumour cells and their spatial proximity to mesenchymal cells. As there is a negative coeffcient associated with this relationship, which tells us that higher HLA class I expression in keratin/tumour cells nearby mesenchymal cell populations lead to poorer survival outcomes for patients.

```{r KM curve state changes, fig.width=5, fig.height=4}
# Selecting the most significant relationship
survRelationship <- stateMat[["Keratin_Tumour__CD4_Cell__Keratin6"]]
survRelationship <- ifelse(survRelationship > median(survRelationship), "Higher expression in close cells", "Lower expression in close cells")

# Plotting Kaplan-Meier curve
survfit2(kerenSurv ~ survRelationship) |>
  ggsurvfit() +
  add_pvalue() +
  ggtitle("Keratin_Tumour__CD4_Cell__Keratin6")
```


# Region analysis using lisaClust

Next we can cluster areas with similar spatial interactions to identify regions using lisaClust. Here we set `k = 5` to identify 5 regions.

```{r lisaClust}
set.seed(51773)

# Preparing features for lisaClust
kerenSCE <- lisaClust::lisaClust(kerenSCE, k = 5)
```

The regions identified by licaClust can be visualised using the `hatchingPlot` function.

```{r hatchingPlot, fig.height=5, fig.width=6.5}
# Use hatching to visualise regions and cell types.
lisaClust::hatchingPlot(kerenSCE,
  useImages = "5",
  line.spacing = 41, # spacing of lines
  nbp = 100 # smoothness of lines
)
```

# Marker Means

`Statial` provides functionality to identify the average marker expression of a given cell type in a given region, using the `getMarkerMeans` function. Similar to the analysis above, these features can also be used for survival analysis.

```{r colTest regions}
cellTypeRegionMeans <- getMarkerMeans(kerenSCE,
  imageID = "imageID",
  cellType = "cellType",
  region = "region"
)

survivalResults <- colTest(cellTypeRegionMeans[names(kerenSurv), ], kerenSurv, type = "survival")

head(survivalResults)
```

# Patient classification

Finally we demonstrate how we can use `ClassifyR` to perform patient classification with the features generated from `Statial`. In addition to the kontextual, state changes, and marker means values, we also calculate cell type proportions and region proportions using the `getProp` function in `spicyR`. Here we perform 3 fold cross validation with 10 repeats, using a CoxPH model for survival classification, feature selection is also performed by selecting the top 10 features per fold using a CoxPH model.

```{r ClassifyR}
# Calculate cell type and region proportions
cellTypeProp <- getProp(kerenSCE,
  feature = "cellType",
  imageID = "imageID"
)
regionProp <- getProp(kerenSCE,
  feature = "region",
  imageID = "imageID"
)

# Combine all the features into a list for classification
featureList <- list(
  states = stateMat,
  kontextual = kontextMat,
  regionMarkerMeans = cellTypeRegionMeans,
  cellTypeProp = cellTypeProp,
  regionProp = regionProp
)

# Ensure the rownames of the features match the order of the survival vector
featureList <- lapply(featureList, function(x) x[names(kerenSurv), ])


set.seed(51773)

kerenCV <- crossValidate(
  measurements = featureList,
  outcome = kerenSurv,
  classifier = "CoxPH",
  selectionMethod = "CoxPH",
  nFolds = 5,
  nFeatures = 10,
  nRepeats = 20,
  nCores = 1
)
```

Here, we use the `performancePlot` function to assess the C-index from each repeat of the 3-fold cross-validation. We can see the resulting C-indexes are very variable due to the dataset only containing 10 images.

```{r performancePlot}
# Calculate AUC for each cross-validation repeat and plot.
performancePlot(kerenCV,
  characteristicsList = list(x = "Assay Name")
) +
  theme(axis.text.x = element_text(angle = 45, vjust = 1, hjust = 1))
```

# References

Keren, L., Bosse, M., Marquez, D., Angoshtari, R., Jain, S., Varma, S., Yang, S. R., Kurian, A., Van Valen, D., West, R., Bendall, S. C., & Angelo, M. (2018). A Structured Tumor-Immune Microenvironment in Triple Negative Breast Cancer Revealed by Multiplexed Ion Beam Imaging. Cell, 174(6), 1373-1387.e1319. ([DOI](https://doi.org/10.1016/j.cell.2018.08.039))

# sessionInfo

```{r sessionInfo}
sessionInfo()
```

```{r}

colData(kerenSCE)$Censored <- colData(kerenSCEVig)$Censored



data("kerenSCE")

kerenImage6 = kerenSCE[, kerenSCE$imageID =="6"]
#'
relabelResult <- relabelKontextual(
  cells = kerenImage6,
  nSim = 5,
  r = 250,
  from = "CD4_Cell",
  to = "Keratin_Tumour",
  parent = c("CD4_Cell", "Macrophages"),
  cores = 2
)
1+1

p <- plotStateChanges(
  cells = kerenSCE,
  type = "distances",
  image = "29",
  from = "Keratin_Tumour",
  to = "CD4_Cell",
  marker = "Keratin6",
  size = 1,
  shape = 19,
  interactive = FALSE,
  plotModelFit = FALSE,
  method = "lm")

p
 kontextMat = prepMatrix(CD4_Kontextual)

```

```{r}
#TODO: Add these functionalities into getDistances
data(headSCE)
intensitiesData <- data.frame(t(assay(headSCE, "intensities")))
spatialData <- data.frame(colData(headSCE))
markersToUse <- colnames(intensitiesData)

rownames(SCE)

colData(SCE)[, factor_cols] <- colData(SCE)[, factor_cols] %>% as.character 

data("kerenSCE")

singleCellDataCounts <- getAbundances(kerenSCE,
   nCores = 30,
   Rs = c(200),
   whichCellTypes = c("Keratin_Tumour", "Macrophages")
 )


#TURN IMAGEID COLUMN INTO A CHARACTER COLUMN

#

# all(!markersToUse %in% colnames(colData(SCE)))
# 
# singleCellData <- cbind(spatialData[rownames(intensitiesData), ], intensitiesData)
# singleCellData <- singleCellData %>%
#   mutate_at(markersToUse, function(x) ifelse(is.na(x), 0, x)) %>% #replace NAs with 0
#   mutate_if(is.factor, as.character)
```

```{r}
colnames(colData(kerenSCE))[40] <- "Survival_days_capped"


colData(kerenSCE) <- colData(kerenSCE) %>% as.data.frame %>% select(c(x, y, CellID, imageID, cellType, tumour_type, DONOR_NO, AGE_AT_DX, Survival_days_capped)) %>% DataFrame()

colData(kerenSCE)$imageID <- colData(kerenSCE)$imageID %>% as.character
#TURN IMAGEID COLUMN INTO A CHARACTER COLUMN

#


colData(kerenSCE)$cellType[colData(kerenSCE)$cellType == "CD4"] <- "CD4_Cell"
colData(kerenSCE)$cellType[colData(kerenSCE)$cellType == "CD8"] <- "CD8_Cell"
colData(kerenSCE)$cellType[colData(kerenSCE)$cellType == "CD3"] <- "CD3_Cell"

data(kerenSCE)
subset <- kerenSCE[, colData(kerenSCE)$imageID %in% c("1", "2", "3")]


#TURN IMAGEID COLUMN INTO A CHARACTER COLUMN

#


SCEsubset <- getDistances(SCEsubset,
                         Rs = c(200),
                         # whichCellTypes = c("Macrophages", "Keratin_Tumour"),
                         nCores = 3
                         )

SCEsubset <- getAbundances(SCEsubset,
                             Rs = c(200),
                             nCores = 3
                             # whichCellTypes = c("Macrophages", "Keratin_Tumour")
                             )

# tmp <- getAbundances(headSCE,
#                              Rs = c(200),
#                              whichCellTypes = c("MC2", "SC7")
#                              )

SCEsubset <- calcContamination(subset,
                         Rs = c(200))

test <- getStateChanges(
  singleCellData = SCEsubset,
  Rs = c(200),
  typeAll = c("dist200", "abundance200"),
  method = "lm",
  isMixed = FALSE,
  cellTypesToModel = "Keratin_Tumour",
  nCores = 3)


test2 <- getStateChanges(
  singleCellData = SCEsubset,
  Rs = c(200),
  typeAll = c("dist200", "abundance200"),
  method = "lm",
  isMixed = FALSE,
  cellTypesToModel = "Keratin_Tumour",
  nCores = 30)


# 

visualiseImageRelationship(
  data = SCEsubset,
  Rs = c(200),
  imageID = "3",
  mainCellType = "CD8_Cell",
  interactingCellType = "CD3_Cell",
  depedentMarker = "CD8",
  interactive = FALSE,
  plotModelFit = FALSE,
  method = "rlm",
  modelType = "dist200_"
)

namesVector <- unique(colData(SCE)$imageID)

classificationDataKeep <- factor(x = rep("cool", length(namesVector))) %>% set_names(namesVector)

modellingData <- listImageModelsCVFormat(imageModels, classificationDataKeep, values_from = "tValue",
                                     removeColsThresh = 0.2,
                                     missingReplacement = 0)

#RUN CLASSIFICATION
interactionsResults =  seq(1, length(modellingData)) %>%
  bplapply(function(x) 
    {
    print(x)
    try({
    interactionsResults = ClassifyR::crossValidate(measurements = modellingData[[x]],
                                            outcome = classificationDataKeep,
                                            nFeatures = 10,
                                            selectionMethod = "t-test",
                                            classifier = "XGB",
                                            nRepeats = 50,
                                            nCores = 1,
                                            characteristicsLabel = names(modellingData)[[x]])
    })
    
  }, BPPARAM = BiocParallel::MulticoreParam(workers = 1))


names(interactionsResults) = names(modellingData)
interactionsResults = interactionsResults[unlist(lapply(interactionsResults,class)) != "try-error"]

#PLOT BOXPLOT OF ACCURACY
interactionsResults %>%
  lapply(function(x) ClassifyR::calcCVperformance(x, performanceType  = "Balanced Accuracy")@performance) %>% 
  lapply(data.frame) %>% 
  mapply(FUN = function(x, name) x %>% set_names(name), x = .,
         name = names(interactionsResults), SIMPLIFY = FALSE) %>% 
  bind_cols() %>% 
  gather() %>% 
  mutate(key = str_replace_all(key, "_", " ")) %>% 
  mutate(key = str_replace_all(key, "\\.", " ")) %>% 
  mutate(key = str_replace_all(key, "None", "Simple")) %>% 
  mutate(key = str_replace_all(key, "count", "abundance")) %>% 
  mutate(key = str_to_title(key)) %>% 
  ggplot(aes(x = reorder(key, value, median), y = value)) + 
  geom_boxplot() +
  labs(y = "Balanced Accuracy", x = "Assay Name") +
  geom_hline(yintercept = 0.5, lty = "dashed") +
  theme(axis.text=element_text(size = 7)) +
  coord_flip()  +
  ggtitle("Head and Neck Data (XGB)")










```

```{r}
# Rs = c(200)
# 
# singleCellDataDistances <- singleCellData %>%
#     split(~imageID) %>%
#     BiocParallel::bplapply(distanceCalculator,
#                            maxRS = max(Rs)
#     )

```


```{r}
# data(headSCE)
# 
# SCE_complete <- getDistances(headSCE,
#                              Rs = c(200)
#                              )
# 
# SCE_complete <- getAbundances(SCE_complete,
#                              Rs = c(200)
#                              )
# 
# SCE_complete <- calcContamination(SCE_complete,
#                                   Rs = c(200)
#                                   )
# 
# imageModelsTest <- getStateChanges(
#   singleCellData = SCE_complete,
#   Rs = c(200),
#   typeAll = c("dist200", "abundance200"),
#   covariates = c("rfMainCellProb"),
#   method = "lm",
#   isMixed = FALSE,
#   nCores = 1)


```

```{r}


```



```{r}
data(headSCE)

library(dplyr)
data("headSCE")
singleCellDataDistances <- getDistances(headSCE,
 nCores = 1,
 Rs = c(200),
 whichCellTypes = c("MC2")
)


SCEhead <- getDistances(headSCE, 
                        Rs = c(200),
                        nCores = 5,
                    whichCellTypes = c("MC2", "SC7"))

tmp <- metadata(SCEhead)$Rs200 %>% as.data.frame

rownames(tmp) <- colnames(SCEhead)
                    
reducedDim(SCEhead, "dist") <- tmp
reducedDim(SCEhead, "abundance") <- tmp

# tmp <- SCEhead@metadata[["dist200"]]

# ttmp <- left_join(tmp, contams)

# tmp <- SCEhead@metadata$dist200_
# SCEhead <- getDistances(SCEhead, Rs = c(100), 
#                     # whichCellTypes = c("MC2", "EC1")
#                     )
SCEhead <- getAbundances(SCEhead, Rs = c(200),
                     whichCellTypes = c("MC2", "SC7")
                     )

# SCEhead <- calcContamination(SCEhead,
#                          Rs = c(200))
# tmp <- as.data.frame(colData(SCEhead))

# visualiseImageRelationship(
#   data = SCEhead,
#   Rs = c(200),
#   imageID = "8",
#   mainCellType = "MC2",
#   interactingCellType = "SC7",
#   depedentMarker = "DNA2",
#   interactive = FALSE,
#   plotModelFit = FALSE,
#   method = "rlm",
#   modelType = "dist200_"
# )
imageModels <- getStateChanges(
  singleCellData = SCEhead,
  Rs = c(200),
  typeAll = c("dist200", "abundance200"),
  method = "lm",
  isMixed = FALSE,
  condition = "condition",
  cellTypesToModel = "MC2",
  nCores = 1)

imageModels <- getStateChanges(
  singleCellData = SCE,
  Rs = c(200),
  typeAll = c("dist200", "abundance200"),
  method = "lm",
  isMixed = TRUE,
  cellTypesToModel = "MC2",
  nCores = 1)

imageModelsFast <- getStateChangesFast(
 singleCellData = SCE,
 Rs = c(200),
 type = c("dist200"),
 nCores = 1
)
# imageModelsNewNoContam <- getStateChanges(
#   singleCellData = SCE,
#   markers = markersToUse,
#   Rs = c(200),
#   typeAll = c("dist200", "abundance200"),
#   method = "lm",
#   # cellTypesToModel = "MC2",
#   nCores = 40)

#SPECIFY FEATURES - HERE IT IS NON PROGRESSIVE AND PROGRESSIVE. THESE ARE NOT FOUND IN HEADSCE.
classificationDataKeep <- factor(x = c("NP", "NP", "NP", "P", "P")) %>% set_names(c("1", "5", "6", "7", "8"))

metadata(SCEhead)$Rs200 <- metadata(SCEhead)$Rs200 %>% mutate(condition = case_when(
  imageID %in% c("1", "5", "6") ~ "NP",
  imageID %in% c("7", "8") ~ "P",
  TRUE ~ "P"
))

modellingData <- listImageModelsCVFormat(imageModelsNew, classificationDataKeep, values_from = "tValue",
                                     removeColsThresh = 0.2,
                                     missingReplacement = 0)

#RUN CLASSIFICATION
interactionsResults =  seq(1, length(modellingData)) %>%
  bplapply(function(x) 
    {
    print(x)
    try({
    interactionsResults = ClassifyR::crossValidate(measurements = modellingData[[x]],
                                            outcome = classificationDataKeep,
                                            nFeatures = 10,
                                            selectionMethod = "t-test",
                                            classifier = "XGB",
                                            nRepeats = 50,
                                            nCores = 1,
                                            characteristicsLabel = names(modellingData)[[x]])
    })
    
  }, BPPARAM = BiocParallel::MulticoreParam(workers = 1))


names(interactionsResults) = names(modellingData)
interactionsResults = interactionsResults[unlist(lapply(interactionsResults,class)) != "try-error"]

#PLOT BOXPLOT OF ACCURACY
interactionsResults %>%
  lapply(function(x) ClassifyR::calcCVperformance(x, performanceType  = "Balanced Accuracy")@performance) %>% 
  lapply(data.frame) %>% 
  mapply(FUN = function(x, name) x %>% set_names(name), x = .,
         name = names(interactionsResults), SIMPLIFY = FALSE) %>% 
  bind_cols() %>% 
  gather() %>% 
  mutate(key = str_replace_all(key, "_", " ")) %>% 
  mutate(key = str_replace_all(key, "\\.", " ")) %>% 
  mutate(key = str_replace_all(key, "None", "Simple")) %>% 
  mutate(key = str_replace_all(key, "count", "abundance")) %>% 
  mutate(key = str_to_title(key)) %>% 
  ggplot(aes(x = reorder(key, value, median), y = value)) + 
  geom_boxplot() +
  labs(y = "Balanced Accuracy", x = "Assay Name") +
  geom_hline(yintercept = 0.5, lty = "dashed") +
  theme(axis.text=element_text(size = 7)) +
  coord_flip()  +
  ggtitle("Head and Neck Data (XGB)")

#PLOT INDIVIDUAL PATIENT HEATMAP
interactionsResults %>% 
  lapply(function(x) ClassifyR::calcCVperformance(x, performanceType  = "Sample Accuracy")) %>% 
  lapply(function(x) data.frame(x@performance)) %>% 
  mapply(FUN = function(x, name) x %>% set_names(name), x = .,
         name = names(interactionsResults), SIMPLIFY = FALSE) %>% 
  bind_cols() %>% 
  heatmaply::heatmaply(show_dendrogram = c(FALSE, FALSE)) 


#ROC PLOT
interactionsResults %>% 
  ROCplot() +
  ggthemes::scale_colour_tableau() +
  labs(colour = "Model")

#Accessing Chosen Features (if Feature Selection was used)
interactionsResults$rfMainCellProb.dist200_@chosenFeatures


#SHAPLY PLOT
dataX <- MatrixModels::model.Matrix(~ 0 + ., data = modellingData$rfMainCellProb.dist200_[, interactionsResults$rfMainCellProb.dist200_@rankedFeatures[[41]] ], sparse = TRUE)
mod = interactionsResults$rfMainCellProb.dist200_@models[[41]]
shp = shapviz(mod, X_pred = data.matrix(dataX[, mod$feature_names]), which_class =  2)

sv_importance(shp, kind = "beeswarm", show_numbers = TRUE) +
  ggtitle("Shapley Feature Importance Plot") 

ggsave("shapleyPlot.png", dpi = 1000, width = 10, height = 8)

```

```{r}
# tmpz <- SCE
# markers <- markersToUse
# seed = 2022
# num.trees = 100
# verbose = FALSE
# missingReplacement = 0
# 
# if ("SingleCellExperiment" %in% class(tmpz)) {
#   tmpzNew <- data.frame(
#     SummarizedExperiment::colData(tmpz)
#   ) %>%
#     dplyr::mutate(
#       imageID = as.character(imageID), cellType = as.character(cellType)
#     )
# 
#   if ("intensities" %in% assayNames(tmpz)) {
#     tmpzNew <- tmpzNew %>%
#       cbind(t(SummarizedExperiment::assay(tmpz, "intensities")))
#     if (is.null(markers)) {
#       markers <- rownames(
#         SummarizedExperiment::assay(tmpz, "intensities")
#       )
#     }
#   }
# 
# 
#   tmpz <- as.data.frame(colData(SCE))
# }
# 
# any(is.na(tmp))

```

```{r}
# 
# tmp <- headSCE
# markers <- markersToUse
# seed = 2022
# num.trees = 100
# verbose = FALSE
# missingReplacement = 0
# 
# if ("SingleCellExperiment" %in% class(tmp)) {
#   tmpNew <- data.frame(
#     SummarizedExperiment::colData(tmp)
#   ) %>%
#     dplyr::mutate(
#       imageID = as.character(imageID), cellType = as.character(cellType)
#     )
# 
#   if ("intensities" %in% assayNames(tmp)) {
#     tmpNew <- tmpNew %>%
#       cbind(t(SummarizedExperiment::assay(tmp, "intensities")))
#     if (is.null(markers)) {
#       markers <- rownames(
#         SummarizedExperiment::assay(tmp, "intensities")
#       )
#     }
#   }
# 
# 
#   tmp <- as.data.frame(colData(SCE))
# }
# 
# rfData <- tmp %>%
#   dplyr::select(cellType, markers) %>%
#   dplyr::mutate(dplyr::across(markers, function(x) ifelse(is.nan(x) | is.na(x), 0, x)))
# 
# set.seed(seed)
# rfModel <- ranger::ranger(
#   as.factor(cellType) ~ .,
#   data = rfData,
#   num.trees = num.trees,
#   probability = TRUE
# )
# 
# if (verbose == TRUE) {
#   print(rfModel)
# }
# 
# predictions <- predict(rfModel, rfData)$predictions
# 
# maxn <- function(n) function(x) order(x, decreasing = TRUE)[!is.na(x)][n]
# 
# rfData <- cbind(rfData, predictions) %>%
#     dplyr::mutate(
#       rfMaxCellProb = apply(
#         .[colnames(predictions)],
#         1,
#         function(x) x[maxn(1)(x)]
#       )
#     ) %>%
#     dplyr::mutate(
#       rfSecondLargestCellProb = apply(
#         .[colnames(predictions)],
#         1,
#         function(x) x[maxn(2)(x)]
#       )
#     ) %>%
#     dplyr::mutate(
#       rfMainCellProb = apply(
#         .[c("cellType", colnames(predictions))],
#         1,
#         function(x) as.numeric(x[x["cellType"]])
#       )
#     ) %>%
#     dplyr::select(-colnames(predictions)) %>%
#     tibble::rownames_to_column("cellID") %>%
#     dplyr::mutate(cellID = stringr::str_replace(cellID, "cellID", ""))
#   
#   tmpz <- tmp %>%
#     dplyr::left_join(rfData)
#   
#   tmpz


```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parentCombinations.R
\name{getParentPhylo}
\alias{getParentPhylo}
\title{Extract parent and all children from a Phylo object}
\usage{
getParentPhylo(phylo_tree)
}
\arguments{
\item{phylo_tree}{a phylo object or a treekoR result.}
}
\value{
A named list of parents and their respective children.
}
\description{
This function takes in a `phylo` object or a `treekoR` result from the 
\code{\link[treekoR]{getClusterTree}} function, and converts its into a 
named list of each and children to input into 
\code{\link[Statial]{parentCombinations}}. 

\bold{Note}: Parent populations with one child will be pruned. Make sure to 
include this cell type in the `all` vector when using 
\code{\link[Statial]{parentCombinations}} to ensure this cell type is included
in pairwise calculations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeWindow.R
\name{makeWindow}
\alias{makeWindow}
\title{Creates a window for a PPP object}
\usage{
makeWindow(data, window = "square", window.length = NULL)
}
\arguments{
\item{data}{A single image data frame from a SingleCellExperiment object or PPP object.}

\item{window}{The shape of window around the regions, can be `square`, `convex` or `concave`}

\item{window.length}{A tuning parameter for controlling the level of concavity when estimating concave windows.}
}
\value{
Creates an `owin` class, representing the observation window for the image.
}
\description{
This function creates a window for a `spatstat::ppp` object, the type of
window can be specified using the `window` argument.
}
\examples{
data <- data.frame(x = rnorm(10), y = rnorm(10))
ow <- makeWindow(data, window = "square")

spatstat.geom::ppp(x = data$x, y = data$y, window = ow)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Statial.R
\name{getDistances}
\alias{getDistances}
\title{Wrapper to calculate pairwise distance between cell types by image}
\usage{
getDistances(
  cells,
  maxDist = NULL,
  imageID = "imageID",
  spatialCoords = c("x", "y"),
  cellType = "cellType",
  redDimName = "distances",
  distFun = "min",
  nCores = 1
)
}
\arguments{
\item{cells}{A dataframe with a cellType column as well as x and y spatial coordinates.
The dataframe must contain a imageID column and cellID (unique cell
identifier's) column as well}

\item{maxDist}{The maximum distance considered.}

\item{imageID}{The name of the colData column that stores in the image ID.}

\item{spatialCoords}{The columns that store the spatial coordinates.}

\item{cellType}{The name of the colData column that stores the cell types.}

\item{redDimName}{The name of the reduced dimension to store the distances in.}

\item{distFun}{What distance function to use. Can be min or abundance.}

\item{nCores}{Number of cores for parallel processing.}
}
\description{
Calculates the euclidean distance from each cell to the nearest cell of each
type
}
\examples{
data("kerenSCE")

kerenSCE <- getDistances(kerenSCE,
  maxDist = 200
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{kerenKontextual}
\alias{kerenKontextual}
\title{Kontextual results from kerenSCE}
\format{
kerenKontextual a kontextual results object.
}
\usage{
data(kerenKontextual)
}
\description{
This is a kontextual results data.frame created using Kontextual on the kerenSCE dataset.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Statial.R
\name{getMarkerMeans}
\alias{getMarkerMeans}
\title{Extract the average expression for all markers for each cell type in each
region defined by lisaClust}
\usage{
getMarkerMeans(
  data,
  imageID = NULL,
  cellType = NULL,
  region = NULL,
  markers = NULL,
  assay = 1,
  replaceVal = 0
)
}
\arguments{
\item{data}{A SingleCellExperiment object with intensities data in the assays slot and
regions information in colData generated by lisaClust.}

\item{imageID}{The colData column that stores the image IDs.}

\item{cellType}{The colData column that store the cell types.}

\item{region}{The colData column that stores the regions.}

\item{markers}{A string vector of markers that proxy a cell's state. If NULL, all markers
will be used.}

\item{assay}{Which assay do you want to use for the expression data.}

\item{replaceVal}{A value to replace missing values with.}
}
\description{
Takes a SingleCellExperiment and outputs a dataframe in a convenient format for
cross validation
}
\examples{
data(kerenSCE)

kerenSCE <- kerenSCE[, kerenSCE$imageID \%in\% c("5", "6")]

regionSCE <- lisaClust::lisaClust(kerenSCE, k = 5)

lisaClustOutput <- getMarkerMeans(regionSCE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Konditional.R
\name{isKontextual}
\alias{isKontextual}
\title{Test whether an object is a kontextualResult}
\usage{
isKontextual(kontextualResult)
}
\arguments{
\item{kontextualResult}{a object to test}
}
\description{
Test whether an object is a kontextualResult
}
\examples{
data <- data.frame()
if (!isKontextual(data)) print("Not a kontextualResult")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relabelKonditional.R
\name{relabelKontextual}
\alias{relabelKontextual}
\alias{relabel}
\title{Cell permutation for Kontextual}
\usage{
relabelKontextual(
  cells,
  nSim = 1,
  r,
  from,
  to,
  parent,
  image = NULL,
  returnImages = FALSE,
  inhom = TRUE,
  edge = FALSE,
  cores = 1,
  spatialCoords = c("x", "y"),
  cellType = "cellType",
  imageID = "imageID",
  ...
)

relabel(image, labels = NULL)
}
\arguments{
\item{cells}{A single image data frame from a SingleCellExperiment object}

\item{nSim}{Number of randomisations which will be calculated.}

\item{r}{Radius to evaluated pairwise relationships between from and to cells.}

\item{from}{The first cell type to be evaluated in the pairwise relationship.}

\item{to}{The second cell type to be evaluated in the pairwise relationship.}

\item{parent}{The parent population of the from cell type (must include from cell type).}

\item{image}{A single image from a Single Cell Experiment object.}

\item{returnImages}{A logical value to indicate whether the function should
return the randomised images along with the Kontextual values.}

\item{inhom}{A logical value indicating whether to account for inhomogeneity.}

\item{edge}{A logical value indicating whether to perform edge correction.}

\item{cores}{Number of cores for parallel processing.}

\item{spatialCoords}{A character vector containing the names of the two
spatial dimansions in the data. Defaults to `c("x", "y")`.}

\item{cellType}{The name of the cell type field in the data. Defualts to
"cellType".}

\item{imageID}{The name of the image ID field in the data. Defualts to
"imageID".}

\item{...}{Any arguments passed into \code{\link[Statial]{Kontextual}}}

\item{labels}{A vector of CellTypes labels to be permuted If NULL all cells
labels will be radomised.}
}
\value{
A data frame containing Kontextual value for each randomised image.
If `returnImages = TRUE` function will return a list with Kontextual values
and the randomised images.

A data frame containing all pairwise cell relationships and their
corresponding parent
}
\description{
Function which randomises specified cells in an image and calculates
the `Kontextual` value. This can be used to estimate the null distribution,
of the parent cell population for significance testing.

This function relabels all specified cells within a single image, to
estimate the null distribution of cell population specified.
}
\examples{
data("kerenSCE")

kerenImage6 <- kerenSCE[, kerenSCE$imageID == "6"]

relabelResult <- relabelKontextual(
  cells = kerenImage6,
  nSim = 5,
  r = 250,
  from = "CD4_Cell",
  to = "Keratin_Tumour",
  parent = c("CD4_Cell", "Macrophages"),
  cores = 2
)

data("kerenSCE")

kerenImage6 <- kerenSCE[, kerenSCE$imageID == "6"]

kerenImage6 <- kerenImage6 |>
  SingleCellExperiment::colData() |>
  data.frame()

# Permute CD8 T cells and T cell labels in the image
relabeledImage <- relabel(kerenImage6, labels = c("p53", "Keratin+Tumour"))
plot(relabeledImage)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Statial.R
\name{plotStateChanges}
\alias{plotStateChanges}
\title{Visualise Cell-Cell Marker Relationships}
\usage{
plotStateChanges(
  cells,
  image,
  from,
  to,
  marker,
  type = "distances",
  assay = 1,
  cellType = "cellType",
  imageID = "imageID",
  spatialCoords = c("x", "y"),
  size = 1,
  shape = 19,
  interactive = FALSE,
  plotModelFit = FALSE,
  method = "lm"
)
}
\arguments{
\item{cells}{A SingleCellExperiment that has had distances already calculated.}

\item{image}{An image to subset to.}

\item{from}{A character indicating the name of the cell type (from the cellType column) whose cell state is being investigated in}

\item{to}{A character indicating the name of the cell type (from the cellType column) who may be influencing the cell state of another cell type}

\item{marker}{The marker of interest.}

\item{type}{The name of the reduced dimension to use for the x-axis.}

\item{assay}{Name of the assay that stores the marker expression.}

\item{cellType}{The name of the column in colData that stores the cell types.}

\item{imageID}{The name of the column in colData that stores the image ids.}

\item{spatialCoords}{The names of the columns in colData that store the spatial coordinates.}

\item{size}{Aesthetic numerical variable determining the size of the displayed cells}

\item{shape}{Aesthetic variable determining the shape grouping of the displayed cells}

\item{interactive}{Logical indicating if the output visualisation should be a interactive (plotly)}

\item{plotModelFit}{Logical indicating if fitted values should be plotted or actual intensities for marker specified. The default is to plot actual intensities}

\item{method}{The method to build the model with. Currently the only option is "lm". However, capabilities may be expanded in the future}
}
\description{
Helper functions to visualise OLS model fits for image based state models
}
\details{
image,
}
\examples{
library(dplyr)
data("kerenSCE")

kerenSCE <- getDistances(kerenSCE)

p <- plotStateChanges(
  cells = kerenSCE,
  type = "distances",
  image = "6",
  from = "Keratin_Tumour",
  to = "Macrophages",
  marker = "p53",
  size = 1,
  shape = 19,
  interactive = FALSE,
  plotModelFit = FALSE,
  method = "lm"
)

p

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Statial.R
\name{calcContamination}
\alias{calcContamination}
\title{Calculate the level of marker contamination of each cell}
\usage{
calcContamination(
  cells,
  markers = NULL,
  num.trees = 100,
  verbose = FALSE,
  missingReplacement = 0,
  assay = "intensities",
  cellType = "cellType",
  redDimName = "contaminations"
)
}
\arguments{
\item{cells}{A SingleCellExperiment or SpatialExperiment with a cellType column as well
as marker intensity information corresponding to each cell.}

\item{markers}{A vector of markers that proxy a cell's state. If NULL, all markers
will be used.}

\item{num.trees}{Number of trees to be used in the random forest classifier}

\item{verbose}{A logical indicating whether information about the final random forest
model should be outputted.}

\item{missingReplacement}{A default value to replace missing marker intensities for classification.}

\item{assay}{The assay in the SingleCellExperiment object that contains the desired
marker expressions.}

\item{cellType}{The name of the column in colData that stores the cell types.}

\item{redDimName}{The redDimName to store the output in the sce.}
}
\description{
Calculates contamination scores using a random forest classification
}
\examples{
data("kerenSCE")

singleCellDataDistancesContam <- calcContamination(
  kerenSCE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Konditional.R
\name{prepMatrix}
\alias{prepMatrix}
\title{Convert Kontextual or state changes result to a matrix for classification}
\usage{
prepMatrix(result, replaceVal = 0, column = NULL, test = NULL)
}
\arguments{
\item{result}{a kontextual or state changes result data.frame.}

\item{replaceVal}{value which NAs are replaced with.}

\item{column}{The column which contains the scores that you want to select.}

\item{test}{A column containing which will be the column names of the expanded matrix.}
}
\description{
Convert Kontextual or state changes result to a matrix for classification
}
\examples{
data("kerenSCE")


CD4_Kontextual <- Kontextual(
  cells = kerenSCE,
  r = 50,
  from = "Macrophages",
  to = "Keratin_Tumour",
  parent = c("Macrophages", "CD4_Cell"),
  image = "6"
)


kontextMat <- prepMatrix(CD4_Kontextual)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Konditional.R
\name{Kontextual}
\alias{Kontextual}
\title{Evaluation of pairwise cell relationships, conditional on a 3rd population.}
\usage{
Kontextual(
  cells,
  r,
  parentDf = NULL,
  from = NULL,
  to = NULL,
  parent = NULL,
  image = NULL,
  inhom = FALSE,
  edgeCorrect = TRUE,
  window = "convex",
  window.length = NA,
  includeOriginal = TRUE,
  spatialCoords = c("x", "y"),
  cellType = "cellType",
  imageID = "imageID",
  cores = 1
)
}
\arguments{
\item{cells}{A SingleCellExperiment, SpatialExperiment or a list of
data.frames containing columns specifying the imageID, cellType, and x and y
spatial coordinates.}

\item{r}{Radii to evaluated pairwise relationships between from and to cells.}

\item{parentDf}{A data frame from \code{\link[Statial]{parentCombinations}}}

\item{from}{The first cell type to be evaluated in the pairwise relationship.}

\item{to}{The second cell type to be evaluated in the pairwise relationship.}

\item{parent}{The parent population of the from cell type (must include from cell type).}

\item{image}{A vector of images to subset the results to. If NULL we default to all images.}

\item{inhom}{A logical value indicating whether to account for inhomogeneity.}

\item{edgeCorrect}{A logical value indicating whether to perform edge correction.}

\item{window}{Type of window for data, either `square`, `convex` or `concave`,
passed into \code{\link[Statial]{makeWindow}}}

\item{window.length}{A tuning parameter for controlling the level of concavity when estimating concave windows.
Passed into \code{\link[Statial]{makeWindow}}}

\item{includeOriginal}{A logical value to return the original L function
values along with the kontextual values.}

\item{spatialCoords}{The columns which contain the x and y spatial coordinates.}

\item{cellType}{The column which contains the cell types.}

\item{imageID}{The column which contains image identifiers.}

\item{cores}{Number of cores for parallel processing.}
}
\value{
A kontextualResult object
}
\description{
Kontextual identifies the relationship between two cell types which are
conditional on the spatial behaviour of a 3rd cell population, for a
particular radius (r).
}
\examples{
# Load data
data("kerenSCE")


CD4_Kontextual <- Kontextual(
  cells = kerenSCE,
  r = 50,
  from = "Macrophages",
  to = "Keratin_Tumour",
  parent = c("Macrophages", "CD4_Cell"),
  image = "6"
)


head(CD4_Kontextual)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Statial.R
\name{calcStateChanges}
\alias{calcStateChanges}
\title{First layer wrapper function to build linear models measuring state changes}
\usage{
calcStateChanges(
  cells,
  marker = NULL,
  from = NULL,
  to = NULL,
  image = NULL,
  type = "distances",
  assay = 1,
  cellType = "cellType",
  imageID = "imageID",
  contamination = NULL,
  test = "g",
  minCells = 20,
  verbose = FALSE,
  timeout = 10,
  nCores = 1
)
}
\arguments{
\item{cells}{A dataframe with a imageID, cellType, and marker intensity column along
with covariates (e.g. distance or abundance of the nearest cell type) to
model cell state changes}

\item{marker}{A vector of markers that proxy a cell's state. If NULL, all markers
will be used.}

\item{from}{A vector of cell types to use as the primary cells. If NULL,
all cell types  will be used.}

\item{to}{A vector of cell types to use as the interacting cells. If NULL,
all cell types  will be used.}

\item{image}{A vector of images to filter to. If null all images will be used.}

\item{type}{What type of state change. This value should be in reduced dimensions.}

\item{assay}{The assay in the SingleCellExperiment object that contains the marker expressions.}

\item{cellType}{The column in colData that stores the cell types.}

\item{imageID}{The column in colData that stores the image ids.}

\item{contamination}{If TRUE, use the contamination scores that have previously
been calculate. Otherwise a name of which reduced dimension contains the scores.}

\item{test}{The type of test to perform. By default this will assume the data 
is Gaussian. A value of "nb" will use a negative binomial to model the expression.}

\item{minCells}{The minimum number of cells required to fit a model.}

\item{verbose}{A logical indicating if messages should be printed}

\item{timeout}{A maximum time allowed to build each model. Setting this may be important
when building rlm mixed linear models}

\item{nCores}{Number of cores for parallel processing}
}
\description{
Builds linear models measuring marker based state changes in a cell type
based of the proximity or abundance of another cell type. The function
provides the option to build robust and mixed linear model variants
}
\examples{
library(dplyr)
data("kerenSCE")

kerenSCE <- kerenSCE[, kerenSCE$imageID \%in\% c(5, 6)]

kerenSCE <- getDistances(kerenSCE,
  maxDist = 200,
)

imageModels <- calcStateChanges(
  cells = kerenSCE,
  from = "Macrophages",
  to = "Tumour"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Statial.R
\name{distanceCalculator}
\alias{distanceCalculator}
\title{Calculate pairwise distance between cell types}
\usage{
distanceCalculator(data, maxDist = 200, distFun = "min")
}
\arguments{
\item{data}{the single cell data of interest}

\item{maxDist}{Maximum distance between pairs of points to be counted as close
pairs.}

\item{distFun}{How to merge duplicate entries.}
}
\description{
Calculates the euclidean distance from each cell to the nearest cell of each
type for a single image
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Statial.R
\name{getAbundances}
\alias{getAbundances}
\title{Wrapper to calculate imhomogenous K function between a cell and surrounding
types on each image}
\usage{
getAbundances(
  cells,
  r = 200,
  distFun = "abundance",
  redDimName = "abundances",
  cellType = "cellType",
  imageID = "imageID",
  spatialCoords = c("x", "y"),
  nCores = 1
)
}
\arguments{
\item{cells}{A dataframe with a cellType column as well as x and y spatial coordinates.
The dataframe must contain a imageID column and cellID (unique cell
identifier's) column as well}

\item{r}{Radius to include in that calculation of pairwise abundance (K-function)
between cells (can be a numeric or vector of radii)}

\item{distFun}{What distance function to use.}

\item{redDimName}{Name of the reduced dimension to store in sce.}

\item{cellType}{The name of the column in colData that stores the cell types.}

\item{imageID}{The name of the column in colData that Stores the image ids.}

\item{spatialCoords}{The names of the columns in colData that store the spatial coordinates.}

\item{nCores}{Number of cores for parallel processing}
}
\description{
Calculate the imhomogenous K function (a measure of cell type abundance) for
each cell to other cell types
}
\examples{
library(dplyr)
data("kerenSCE")

singleCellDataCounts <- getAbundances(kerenSCE,
  r = 200,
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rsCurve.R
\name{kontextCurve}
\alias{kontextCurve}
\title{Evaluation of Kontextual over a range of radii.}
\usage{
kontextCurve(
  cells,
  from,
  to,
  parent,
  image = NULL,
  rs = seq(10, 100, 10),
  inhom = FALSE,
  edge = TRUE,
  se = FALSE,
  nSim = 20,
  cores = 1,
  imageID = "imageID",
  cellType = "cellType",
  ...
)
}
\arguments{
\item{cells}{A single image from a SingleCellExperiment object}

\item{from}{The first cell type to be evaluated in the pairwise relationship.}

\item{to}{The second cell type to be evaluated in the pairwise relationship.}

\item{parent}{The parent population of the from cell type (must include from cell type).}

\item{image}{A vector of images to subset the results to. If NULL we default to all images.}

\item{rs}{A vector of radii to evaluate kontextual over.}

\item{inhom}{A logical value indicating whether to perform an inhomogeneous L function.}

\item{edge}{A logical value indicating whether to perform edge correction.}

\item{se}{A logical value to indicate if the standard deviation of
kontextual should be calculated to construct error bars.}

\item{nSim}{Number of randomisations to perform using \code{\link[Statial]{relabelKontextual}},
which will be used to calculated the SE.}

\item{cores}{Number of cores for parallel processing.}

\item{imageID}{The column in colData that stores the image ids.}

\item{cellType}{The column in colData that stores the cell types.}

\item{...}{Any arguments passed into \code{\link[Statial]{Kontextual}}.}
}
\value{
A data frame of original L values and Kontextual values evaluated
over a range of radii.
}
\description{
This function obtains `Kontextual` values over a range of radii, standard
deviations for each value can be obtained using permutation for significance
testing. To obtain estimates for standard deviations specify `se = TRUE`.
}
\examples{

data("kerenSCE")

kerenImage6 <- kerenSCE[, kerenSCE$imageID == "6"]

rsDf <- kontextCurve(
  cells = kerenSCE,
  from = "CD4_Cell",
  to = "Keratin_Tumour",
  parent = c("CD4_Cell", "Macrophages"),
  rs = seq(10, 510, 100),
  cores = 2
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parentCombinations.R
\name{parentCombinations}
\alias{parentCombinations}
\title{Create all combinations of cell type relationships from a list of parents}
\usage{
parentCombinations(all, ..., parentList = NULL)
}
\arguments{
\item{all}{A list of all the `to` cell types Kontextual is evaluated over.}

\item{...}{Vectors of each parent population.}

\item{parentList}{a named list where the names correspond to parent names and
the values contain a vector of children for that parent. Note: If parentList
is specified the `...` argument will be ignored, see examples.}
}
\value{
A data frame containing all pairwise cell relationships and their
corresponding parent
}
\description{
This function takes in named vectors of all the parent populations in the
dataset, and creates a data frame containing all pairwise cell relationships,
this data frame can be inputed into the `parentDf` argument in `Kontextual`.
}
\examples{
# Example 1, using `parentList`

parentList <- list(
  "tcells" = c("CD4", "CD8"),
  "tissue" = c("epithelial", "stromal")
)

allCells <- c("tumour", "CD4", "CD8", "epithelial", "stromal")

parentCombinations(all = allCells, parentList = parentList)


# Example 2, with `...` operator
tcells <- c("CD4", "CD8")
tissue <- c("epithelial", "stromal")
allCells <- c("tumour", tissue, tcells)

parentCombinations(all = allCells, tcells, tissue)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rsCurve.R
\name{kontextPlot}
\alias{kontextPlot}
\title{Plotting the original and kontextual L values over a range of radii.}
\usage{
kontextPlot(rsDf)
}
\arguments{
\item{rsDf}{A data frame from \code{\link[Statial]{kontextCurve}}.}
}
\value{
A ggplotly object showing the original and kontextual L function
 values over a range of radii
}
\description{
This function takes outputs from rsCurve and plots
them in ggplot. If standard deviation is estimated in rsCurve,
then confidence intervals will be constructed based on the standard deviation.
If the confidence interval overlaps with 0, then the relationship is insignificant
for that radius.
}
\examples{
data("kerenSCE")

kerenImage6 <- kerenSCE[, kerenSCE$imageID == "6"]

rsDf <- kontextCurve(
  cells = kerenImage6,
  from = "p53",
  to = "Immune",
  parent = c("p53", "Keratin+Tumour"),
  rs = seq(10, 510, 100),
  cores = 2
)

kontextPlot(rsDf)

}
